@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -18,7 +18,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  */
-
 package com.github.javaparser;
 
 import com.github.javaparser.ast.CompilationUnit;
@@ -26,6 +25,7 @@ import com.github.javaparser.ast.ImportDeclaration;
 import com.github.javaparser.ast.Node;
 import com.github.javaparser.ast.PackageDeclaration;
 import com.github.javaparser.ast.body.BodyDeclaration;
+import com.github.javaparser.ast.body.MethodDeclaration;
 import com.github.javaparser.ast.body.Parameter;
 import com.github.javaparser.ast.body.TypeDeclaration;
 import com.github.javaparser.ast.expr.*;
@@ -38,6 +38,8 @@ import com.github.javaparser.ast.type.ClassOrInterfaceType;
 import com.github.javaparser.ast.type.Type;
 import com.github.javaparser.ast.type.TypeParameter;
 import com.github.javaparser.javadoc.Javadoc;
+import com.github.javaparser.quality.NotNull;
+import com.github.javaparser.quality.Preconditions;
 
 import java.io.*;
 import java.nio.charset.Charset;
@@ -47,28 +49,34 @@ import java.nio.file.Path;
  * A simpler, static API than {@link JavaParser}.
  */
 public final class StaticJavaParser {
-    private static ParserConfiguration configuration = new ParserConfiguration();
 
-    private StaticJavaParser() {
+    // use ThreadLocal to resolve possible concurrency issues.
+    private static final ThreadLocal<ParserConfiguration> localConfiguration = ThreadLocal.withInitial(ParserConfiguration::new);
+
+    /**
+     * Get the configuration for the parse... methods. Deprecated method.
+     *
+     * @deprecated use {@link #getParserConfiguration()} instead
+     */
+    @Deprecated
+    public static ParserConfiguration getConfiguration() {
+        return getParserConfiguration();
     }
 
     /**
      * Get the configuration for the parse... methods.
      */
-    public static ParserConfiguration getConfiguration() {
-        return configuration;
+    public static ParserConfiguration getParserConfiguration() {
+        return localConfiguration.get();
     }
 
     /**
      * Set the configuration for the static parse... methods.
      * This is a STATIC field, so modifying it will directly change how all static parse... methods work!
      */
-    public static void setConfiguration(ParserConfiguration configuration) {
-        StaticJavaParser.configuration = configuration;
-    }
-
-    private static JavaParser newParser() {
-        return new JavaParser(configuration);
+    public static void setConfiguration(@NotNull ParserConfiguration configuration) {
+        Preconditions.checkNotNull(configuration, "Parameter configuration can't be null.");
+        localConfiguration.set(configuration);
     }
 
     /**
@@ -82,7 +90,9 @@ public final class StaticJavaParser {
      * @deprecated set the encoding in the {@link ParserConfiguration}
      */
     @Deprecated
-    public static CompilationUnit parse(final InputStream in, Charset encoding) {
+    public static CompilationUnit parse(@NotNull final InputStream in, @NotNull Charset encoding) {
+        Preconditions.checkNotNull(in, "Parameter in can't be null.");
+        Preconditions.checkNotNull(encoding, "Parameter encoding can't be null.");
         return handleResult(newParser().parse(in, encoding));
     }
 
@@ -94,8 +104,9 @@ public final class StaticJavaParser {
      * @return CompilationUnit representing the Java source code
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static CompilationUnit parse(final InputStream in) {
-        return handleResult(newParser().parse(in));
+    public static CompilationUnit parse(@NotNull final InputStream in) {
+        Preconditions.checkNotNull(in, "Parameter in can't be null.");
+        return newParserAdapted().parse(in);
     }
 
     /**
@@ -110,7 +121,9 @@ public final class StaticJavaParser {
      * @deprecated set the encoding in the {@link ParserConfiguration}
      */
     @Deprecated
-    public static CompilationUnit parse(final File file, final Charset encoding) throws FileNotFoundException {
+    public static CompilationUnit parse(@NotNull final File file, @NotNull final Charset encoding) throws FileNotFoundException {
+        Preconditions.checkNotNull(file, "Parameter file can't be null.");
+        Preconditions.checkNotNull(encoding, "Parameter encoding can't be null.");
         return handleResult(newParser().parse(file, encoding));
     }
 
@@ -123,8 +136,9 @@ public final class StaticJavaParser {
      * @throws ParseProblemException if the source code has parser errors
      * @throws FileNotFoundException the file was not found
      */
-    public static CompilationUnit parse(final File file) throws FileNotFoundException {
-        return handleResult(newParser().parse(file));
+    public static CompilationUnit parse(@NotNull final File file) throws FileNotFoundException {
+        Preconditions.checkNotNull(file, "Parameter file can't be null.");
+        return newParserAdapted().parse(file);
     }
 
     /**
@@ -139,7 +153,9 @@ public final class StaticJavaParser {
      * @deprecated set the encoding in the {@link ParserConfiguration}
      */
     @Deprecated
-    public static CompilationUnit parse(final Path path, final Charset encoding) throws IOException {
+    public static CompilationUnit parse(@NotNull final Path path, @NotNull final Charset encoding) throws IOException {
+        Preconditions.checkNotNull(path, "Parameter path can't be null.");
+        Preconditions.checkNotNull(encoding, "Parameter encoding can't be null.");
         return handleResult(newParser().parse(path, encoding));
     }
 
@@ -152,8 +168,9 @@ public final class StaticJavaParser {
      * @throws ParseProblemException if the source code has parser errors
      * @throws IOException the path could not be accessed
      */
-    public static CompilationUnit parse(final Path path) throws IOException {
-        return handleResult(newParser().parse(path));
+    public static CompilationUnit parse(@NotNull final Path path) throws IOException {
+        Preconditions.checkNotNull(path, "Parameter path can't be null.");
+        return newParserAdapted().parse(path);
     }
 
     /**
@@ -166,8 +183,9 @@ public final class StaticJavaParser {
      * @throws ParseProblemException if the source code has parser errors
      * @throws IOException the path could not be accessed
      */
-    public static CompilationUnit parseResource(final String path) throws IOException {
-        return handleResult(newParser().parseResource(path));
+    public static CompilationUnit parseResource(@NotNull final String path) throws IOException {
+        Preconditions.checkNotNull(path, "Parameter path can't be null.");
+        return newParserAdapted().parseResource(path);
     }
 
     /**
@@ -183,7 +201,9 @@ public final class StaticJavaParser {
      * @deprecated set the encoding in the {@link ParserConfiguration}
      */
     @Deprecated
-    public static CompilationUnit parseResource(final String path, Charset encoding) throws IOException {
+    public static CompilationUnit parseResource(@NotNull final String path, @NotNull Charset encoding) throws IOException {
+        Preconditions.checkNotNull(path, "Parameter path can't be null.");
+        Preconditions.checkNotNull(encoding, "Parameter encoding can't be null.");
         return handleResult(newParser().parseResource(path, encoding));
     }
 
@@ -200,7 +220,10 @@ public final class StaticJavaParser {
      * @deprecated set the encoding in the {@link ParserConfiguration}
      */
     @Deprecated
-    public static CompilationUnit parseResource(final ClassLoader classLoader, final String path, Charset encoding) throws IOException {
+    public static CompilationUnit parseResource(@NotNull final ClassLoader classLoader, @NotNull final String path, @NotNull Charset encoding) throws IOException {
+        Preconditions.checkNotNull(classLoader, "Parameter classLoader can't be null.");
+        Preconditions.checkNotNull(path, "Parameter path can't be null.");
+        Preconditions.checkNotNull(encoding, "Parameter encoding can't be null.");
         return handleResult(newParser().parseResource(classLoader, path, encoding));
     }
 
@@ -212,8 +235,9 @@ public final class StaticJavaParser {
      * @return CompilationUnit representing the Java source code
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static CompilationUnit parse(final Reader reader) {
-        return handleResult(newParser().parse(reader));
+    public static CompilationUnit parse(@NotNull final Reader reader) {
+        Preconditions.checkNotNull(reader, "Parameter reader can't be null.");
+        return newParserAdapted().parse(reader);
     }
 
     /**
@@ -224,8 +248,9 @@ public final class StaticJavaParser {
      * @return CompilationUnit representing the Java source code
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static CompilationUnit parse(String code) {
-        return handleResult(newParser().parse(code));
+    public static CompilationUnit parse(@NotNull String code) {
+        Preconditions.checkNotNull(code, "Parameter code can't be null.");
+        return newParserAdapted().parse(code);
     }
 
     /**
@@ -236,8 +261,9 @@ public final class StaticJavaParser {
      * @return BlockStmt representing the Java block
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static BlockStmt parseBlock(final String blockStatement) {
-        return handleResult(newParser().parseBlock(blockStatement));
+    public static BlockStmt parseBlock(@NotNull final String blockStatement) {
+        Preconditions.checkNotNull(blockStatement, "Parameter blockStatement can't be null.");
+        return newParserAdapted().parseBlock(blockStatement);
     }
 
     /**
@@ -248,15 +274,9 @@ public final class StaticJavaParser {
      * @return Statement representing the Java statement
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static Statement parseStatement(final String statement) {
-        return handleResult(newParser().parseStatement(statement));
-    }
-
-    private static <T extends Node> T handleResult(ParseResult<T> result) {
-        if (result.isSuccessful()) {
-            return result.getResult().get();
-        }
-        throw new ParseProblemException(result.getProblems());
+    public static Statement parseStatement(@NotNull final String statement) {
+        Preconditions.checkNotNull(statement, "Parameter statement can't be null.");
+        return newParserAdapted().parseStatement(statement);
     }
 
     /**
@@ -267,8 +287,9 @@ public final class StaticJavaParser {
      * @return ImportDeclaration representing the Java import declaration
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static ImportDeclaration parseImport(final String importDeclaration) {
-        return handleResult(newParser().parseImport(importDeclaration));
+    public static ImportDeclaration parseImport(@NotNull final String importDeclaration) {
+        Preconditions.checkNotNull(importDeclaration, "Parameter importDeclaration can't be null.");
+        return newParserAdapted().parseImport(importDeclaration);
     }
 
     /**
@@ -279,8 +300,9 @@ public final class StaticJavaParser {
      * @return Expression representing the Java expression
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static <T extends Expression> T parseExpression(final String expression) {
-        return handleResult(newParser().parseExpression(expression));
+    public static <T extends Expression> T parseExpression(@NotNull final String expression) {
+        Preconditions.checkNotNull(expression, "Parameter expression can't be null.");
+        return newParserAdapted().parseExpression(expression);
     }
 
     /**
@@ -291,8 +313,9 @@ public final class StaticJavaParser {
      * @return AnnotationExpr representing the Java annotation
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static AnnotationExpr parseAnnotation(final String annotation) {
-        return handleResult(newParser().parseAnnotation(annotation));
+    public static AnnotationExpr parseAnnotation(@NotNull final String annotation) {
+        Preconditions.checkNotNull(annotation, "Parameter annotation can't be null.");
+        return newParserAdapted().parseAnnotation(annotation);
     }
 
     /**
@@ -303,8 +326,9 @@ public final class StaticJavaParser {
      * @return BodyDeclaration representing the Java annotation
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static BodyDeclaration<?> parseAnnotationBodyDeclaration(final String body) {
-        return handleResult(newParser().parseAnnotationBodyDeclaration(body));
+    public static BodyDeclaration<?> parseAnnotationBodyDeclaration(@NotNull final String body) {
+        Preconditions.checkNotNull(body, "Parameter body can't be null.");
+        return newParserAdapted().parseAnnotationBodyDeclaration(body);
     }
 
     /**
@@ -315,8 +339,9 @@ public final class StaticJavaParser {
      * @return BodyDeclaration representing the Java interface body
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static BodyDeclaration<?> parseBodyDeclaration(String body) {
-        return handleResult(newParser().parseBodyDeclaration(body));
+    public static BodyDeclaration<?> parseBodyDeclaration(@NotNull String body) {
+        Preconditions.checkNotNull(body, "Parameter body can't be null.");
+        return newParserAdapted().parseBodyDeclaration(body);
     }
 
     /**
@@ -326,8 +351,9 @@ public final class StaticJavaParser {
      * @return ClassOrInterfaceType representing the type
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static ClassOrInterfaceType parseClassOrInterfaceType(String type) {
-        return handleResult(newParser().parseClassOrInterfaceType(type));
+    public static ClassOrInterfaceType parseClassOrInterfaceType(@NotNull String type) {
+        Preconditions.checkNotNull(type, "Parameter type can't be null.");
+        return newParserAdapted().parseClassOrInterfaceType(type);
     }
 
     /**
@@ -337,31 +363,34 @@ public final class StaticJavaParser {
      * @return ClassOrInterfaceType representing the type
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static Type parseType(String type) {
-        return handleResult(newParser().parseType(type));
+    public static Type parseType(@NotNull String type) {
+        Preconditions.checkNotNull(type, "Parameter type can't be null.");
+        return newParserAdapted().parseType(type);
     }
 
     /**
      * Parses a variable declaration expression and returns a {@link VariableDeclarationExpr}
      * that represents it.
      *
-     * @param declaration a variable declaration like <code>int x=2;</code>
+     * @param declaration a variable declaration like {@code int x=2;}
      * @return VariableDeclarationExpr representing the type
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static VariableDeclarationExpr parseVariableDeclarationExpr(String declaration) {
-        return handleResult(newParser().parseVariableDeclarationExpr(declaration));
+    public static VariableDeclarationExpr parseVariableDeclarationExpr(@NotNull String declaration) {
+        Preconditions.checkNotNull(declaration, "Parameter declaration can't be null.");
+        return newParserAdapted().parseVariableDeclarationExpr(declaration);
     }
 
     /**
      * Parses the content of a JavadocComment and returns a {@link Javadoc} that
      * represents it.
      *
-     * @param content a variable declaration like <code>content of my javadoc\n * second line\n * third line</code>
+     * @param content a variable declaration like {@code content of my javadoc\n * second line\n * third line}
      * @return Javadoc representing the content of the comment
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static Javadoc parseJavadoc(String content) {
+    public static Javadoc parseJavadoc(@NotNull String content) {
+        Preconditions.checkNotNull(content, "Parameter content can't be null.");
         return JavadocParser.parse(content);
     }
 
@@ -372,8 +401,9 @@ public final class StaticJavaParser {
      * @return the AST for the statement.
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static ExplicitConstructorInvocationStmt parseExplicitConstructorInvocationStmt(String statement) {
-        return handleResult(newParser().parseExplicitConstructorInvocationStmt(statement));
+    public static ExplicitConstructorInvocationStmt parseExplicitConstructorInvocationStmt(@NotNull String statement) {
+        Preconditions.checkNotNull(statement, "Parameter statement can't be null.");
+        return newParserAdapted().parseExplicitConstructorInvocationStmt(statement);
     }
 
     /**
@@ -383,8 +413,9 @@ public final class StaticJavaParser {
      * @return the AST for the name
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static Name parseName(String qualifiedName) {
-        return handleResult(newParser().parseName(qualifiedName));
+    public static Name parseName(@NotNull String qualifiedName) {
+        Preconditions.checkNotNull(qualifiedName, "Parameter qualifiedName can't be null.");
+        return newParserAdapted().parseName(qualifiedName);
     }
 
     /**
@@ -394,8 +425,9 @@ public final class StaticJavaParser {
      * @return the AST for the name
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static SimpleName parseSimpleName(String name) {
-        return handleResult(newParser().parseSimpleName(name));
+    public static SimpleName parseSimpleName(@NotNull String name) {
+        Preconditions.checkNotNull(name, "Parameter name can't be null.");
+        return newParserAdapted().parseSimpleName(name);
     }
 
     /**
@@ -405,8 +437,9 @@ public final class StaticJavaParser {
      * @return the AST for the parameter
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static Parameter parseParameter(String parameter) {
-        return handleResult(newParser().parseParameter(parameter));
+    public static Parameter parseParameter(@NotNull String parameter) {
+        Preconditions.checkNotNull(parameter, "Parameter parameter can't be null.");
+        return newParserAdapted().parseParameter(parameter);
     }
 
     /**
@@ -416,8 +449,9 @@ public final class StaticJavaParser {
      * @return the AST for the parameter
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static PackageDeclaration parsePackageDeclaration(String packageDeclaration) {
-        return handleResult(newParser().parsePackageDeclaration(packageDeclaration));
+    public static PackageDeclaration parsePackageDeclaration(@NotNull String packageDeclaration) {
+        Preconditions.checkNotNull(packageDeclaration, "Parameter packageDeclaration can't be null.");
+        return newParserAdapted().parsePackageDeclaration(packageDeclaration);
     }
 
     /**
@@ -427,8 +461,9 @@ public final class StaticJavaParser {
      * @return the AST for the type declaration
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static TypeDeclaration<?> parseTypeDeclaration(String typeDeclaration) {
-        return handleResult(newParser().parseTypeDeclaration(typeDeclaration));
+    public static TypeDeclaration<?> parseTypeDeclaration(@NotNull String typeDeclaration) {
+        Preconditions.checkNotNull(typeDeclaration, "Parameter typeDeclaration can't be null.");
+        return newParserAdapted().parseTypeDeclaration(typeDeclaration);
     }
 
     /**
@@ -439,8 +474,9 @@ public final class StaticJavaParser {
      * @throws ParseProblemException if the source code has parser errors
      * @see ModuleDeclaration
      */
-    public static ModuleDeclaration parseModuleDeclaration(String moduleDeclaration) {
-        return handleResult(newParser().parseModuleDeclaration(moduleDeclaration));
+    public static ModuleDeclaration parseModuleDeclaration(@NotNull String moduleDeclaration) {
+        Preconditions.checkNotNull(moduleDeclaration, "Parameter moduleDeclaration can't be null.");
+        return newParserAdapted().parseModuleDeclaration(moduleDeclaration);
     }
 
     /**
@@ -451,11 +487,11 @@ public final class StaticJavaParser {
      * @throws ParseProblemException if the source code has parser errors
      * @see ModuleDirective
      */
-    public static ModuleDirective parseModuleDirective(String moduleDirective) {
-        return handleResult(newParser().parseModuleDirective(moduleDirective));
+    public static ModuleDirective parseModuleDirective(@NotNull String moduleDirective) {
+        Preconditions.checkNotNull(moduleDirective, "Parameter moduleDirective can't be null.");
+        return newParserAdapted().parseModuleDirective(moduleDirective);
     }
 
-
     /**
      * Parses a type parameter and returns it as a TypeParameter
      *
@@ -463,8 +499,56 @@ public final class StaticJavaParser {
      * @return the AST for the type parameter
      * @throws ParseProblemException if the source code has parser errors
      */
-    public static TypeParameter parseTypeParameter(String typeParameter) {
-        return handleResult(newParser().parseTypeParameter(typeParameter));
+    public static TypeParameter parseTypeParameter(@NotNull String typeParameter) {
+        Preconditions.checkNotNull(typeParameter, "Parameter typeParameter can't be null.");
+        return newParserAdapted().parseTypeParameter(typeParameter);
+    }
+
+    /**
+     * Parses a method declaration and returns it as a MethodDeclaration.
+     *
+     * @param methodDeclaration a method declaration like "void foo() {}"
+     * @return the AST for the method declaration
+     * @throws ParseProblemException if the source code has parser errors
+     * @see MethodDeclaration
+     */
+    public static MethodDeclaration parseMethodDeclaration(@NotNull String methodDeclaration) {
+        Preconditions.checkNotNull(methodDeclaration, "Parameter methodDeclaration can't be null.");
+        return newParserAdapted().parseMethodDeclaration(methodDeclaration);
+    }
+
+    /**
+     * Parses an array initializer expression and returns it as ArrayInitializerExpr.
+     *
+     * @param arrayInitializerExpr an array initializer like "{1,2,3}"
+     * @return the AST for the array initializer expression
+     * @throws ParseProblemException if the source code has parser errors
+     * @see ArrayInitializerExpr
+     */
+    public static ArrayInitializerExpr parseArrayInitializerExpr(@NotNull String arrayInitializerExpr) {
+        Preconditions.checkNotNull(arrayInitializerExpr, "Parameter arrayInitializerExpr can't be null.");
+        return newParserAdapted().parseArrayInitializerExpr(arrayInitializerExpr);
+    }
+
+    // Private methods
+
+    private static JavaParser newParser() {
+        return new JavaParser(getParserConfiguration());
+    }
+
+    private static JavaParserAdapter newParserAdapted() {
+        return new JavaParserAdapter(newParser());
+    }
+
+    @Deprecated
+    private static <T extends Node> T handleResult(ParseResult<T> result) {
+        if (result.isSuccessful()) {
+            return result.getResult().get();
+        }
+        throw new ParseProblemException(result.getProblems());
+    }
+
+    private StaticJavaParser() {
     }
 
 }
