@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -18,7 +18,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  */
-
 package com.github.javaparser.ast;
 
 /**
@@ -30,10 +29,7 @@ package com.github.javaparser.ast;
  */
 public enum AccessSpecifier {
 
-    PUBLIC("public"),
-    PRIVATE("private"),
-    PROTECTED("protected"),
-    PACKAGE_PRIVATE("");
+    PUBLIC("public"), PRIVATE("private"), PROTECTED("protected"), NONE("");
 
     private String codeRepresenation;
 
