@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -21,10 +21,7 @@
 package com.github.javaparser.ast;
 
 import com.github.javaparser.*;
-import com.github.javaparser.ast.body.AnnotationDeclaration;
-import com.github.javaparser.ast.body.ClassOrInterfaceDeclaration;
-import com.github.javaparser.ast.body.EnumDeclaration;
-import com.github.javaparser.ast.body.TypeDeclaration;
+import com.github.javaparser.ast.body.*;
 import com.github.javaparser.ast.comments.Comment;
 import com.github.javaparser.ast.comments.JavadocComment;
 import com.github.javaparser.ast.expr.Name;
@@ -38,32 +35,33 @@ import com.github.javaparser.metamodel.CompilationUnitMetaModel;
 import com.github.javaparser.metamodel.InternalProperty;
 import com.github.javaparser.metamodel.JavaParserMetaModel;
 import com.github.javaparser.metamodel.OptionalProperty;
-import com.github.javaparser.printer.PrettyPrinter;
+import com.github.javaparser.printer.Printer;
+import com.github.javaparser.printer.configuration.PrinterConfiguration;
 import com.github.javaparser.utils.ClassUtils;
 import com.github.javaparser.utils.CodeGenerationUtils;
 import com.github.javaparser.utils.Utils;
 
 import java.io.IOException;
+import java.nio.charset.Charset;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.List;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.function.Function;
+import java.util.stream.Collectors;
 
-import static com.github.javaparser.JavaToken.Kind.*;
+import static com.github.javaparser.JavaToken.Kind.EOF;
 import static com.github.javaparser.Providers.UTF8;
 import static com.github.javaparser.Providers.provider;
-import static com.github.javaparser.Range.*;
+import static com.github.javaparser.Range.range;
 import static com.github.javaparser.StaticJavaParser.parseImport;
 import static com.github.javaparser.StaticJavaParser.parseName;
 import static com.github.javaparser.ast.Modifier.createModifierList;
 import static com.github.javaparser.utils.CodeGenerationUtils.subtractPaths;
 import static com.github.javaparser.utils.Utils.assertNotNull;
 
-import com.github.javaparser.ast.Node;
-import com.github.javaparser.ast.Generated;
-
 /**
  * <p>
  * This class represents the entire compilation unit. Each java file denotes a
@@ -81,6 +79,8 @@ import com.github.javaparser.ast.Generated;
  */
 public class CompilationUnit extends Node {
 
+    private static final String JAVA_LANG = "java.lang";
+
     @OptionalProperty
     private PackageDeclaration packageDeclaration;
 
@@ -132,6 +132,49 @@ public class CompilationUnit extends Node {
         v.visit(this, arg);
     }
 
+    /**
+     * Declare a specific printer
+     */
+    public CompilationUnit printer(Printer printer) {
+        setData(PRINTER_KEY, printer);
+        return this;
+    }
+
+    /*
+     * If there is no declared printer, returns a new default printer else returns a new printer with the current configuration
+     */
+    @Override
+    protected Printer getPrinter() {
+        if (!containsData(PRINTER_KEY)) {
+            // create a default printer
+            Printer printer = createDefaultPrinter();
+            printer(printer);
+        }
+        return getData(PRINTER_KEY);
+    }
+
+    /*
+     * Return the printer initialized with the specified configuration
+     */
+    @Override
+    protected Printer getPrinter(PrinterConfiguration config) {
+        Printer printer = getPrinter().setConfiguration(config);
+        printer(printer);
+        return printer;
+    }
+
+    /**
+     * @deprecated getComments was a too generic name and it could be confused with getComment
+     * or getAllContainedComments
+     * Use {@link #getAllComments()} instead
+     */
+    @Deprecated
+    public List<Comment> getComments() {
+        List<Comment> comments = this.getAllContainedComments();
+        this.getComment().ifPresent(comments::add);
+        return comments;
+    }
+
     /**
      * Return a list containing all comments declared in this compilation unit.
      * Including javadocs, line comments and block comments of all types,
@@ -143,15 +186,17 @@ public class CompilationUnit extends Node {
      * @see com.github.javaparser.ast.comments.LineComment
      * @see com.github.javaparser.ast.comments.BlockComment
      */
-    public List<Comment> getComments() {
-        return this.getAllContainedComments();
+    public List<Comment> getAllComments() {
+        List<Comment> comments = this.getAllContainedComments();
+        this.getComment().ifPresent(comments::add);
+        return comments;
     }
 
     /**
      * Retrieves the list of imports declared in this compilation unit or
-     * <code>null</code> if there is no import.
+     * {@code null} if there is no import.
      *
-     * @return the list of imports or <code>none</code> if there is no import
+     * @return the list of imports or {@code none} if there is no import
      */
     @Generated("com.github.javaparser.generator.core.node.PropertyGenerator")
     public NodeList<ImportDeclaration> getImports() {
@@ -165,9 +210,9 @@ public class CompilationUnit extends Node {
     /**
      * Retrieves the package declaration of this compilation unit.<br>
      * If this compilation unit has no package declaration (default package),
-     * <code>Optional.none()</code> is returned.
+     * {@code Optional.none()} is returned.
      *
-     * @return the package declaration or <code>none</code>
+     * @return the package declaration or {@code none}
      */
     @Generated("com.github.javaparser.generator.core.node.PropertyGenerator")
     public Optional<PackageDeclaration> getPackageDeclaration() {
@@ -176,9 +221,9 @@ public class CompilationUnit extends Node {
 
     /**
      * Return the list of top level types declared in this compilation unit.<br>
-     * If there are no types declared, <code>none</code> is returned.
+     * If there are no types declared, {@code none} is returned.
      *
-     * @return the list of types or <code>none</code> null if there is no type
+     * @return the list of types or {@code none} null if there is no type
      * @see AnnotationDeclaration
      * @see ClassOrInterfaceDeclaration
      * @see EnumDeclaration
@@ -189,8 +234,8 @@ public class CompilationUnit extends Node {
     }
 
     /**
-     * Convenience method that wraps <code>getTypes()</code>.<br>
-     * If <code>i</code> is out of bounds, throws <code>IndexOutOfBoundsException.</code>
+     * Convenience method that wraps {@code getTypes()}.<br>
+     * If {@code i} is out of bounds, throws <code>IndexOutOfBoundsException.</code>
      *
      * @param i the index of the type declaration to retrieve
      */
@@ -200,7 +245,7 @@ public class CompilationUnit extends Node {
 
     /**
      * Sets the list of imports of this compilation unit. The list is initially
-     * <code>null</code>.
+     * {@code null}.
      *
      * @param imports the list of imports
      */
@@ -208,7 +253,7 @@ public class CompilationUnit extends Node {
     public CompilationUnit setImports(final NodeList<ImportDeclaration> imports) {
         assertNotNull(imports);
         if (imports == this.imports) {
-            return (CompilationUnit) this;
+            return this;
         }
         notifyPropertyChange(ObservableProperty.IMPORTS, this.imports, imports);
         if (this.imports != null)
@@ -223,22 +268,57 @@ public class CompilationUnit extends Node {
         return this;
     }
 
+    /**
+     * adds an import if not implicitly imported by java (i.e. java.lang) or
+     * added before. Asterisk imports overrule the other imports within the same package.
+     *
+     * @param importDeclaration
+     * @return {@code this}
+     */
     public CompilationUnit addImport(ImportDeclaration importDeclaration) {
-        if (getImports().stream().noneMatch(im -> im.toString().equals(importDeclaration.toString()))) {
+        if (importDeclaration.isAsterisk()) {
+            getImports().removeIf(im -> Objects.equals(getImportPackageName(im).get(), getImportPackageName(importDeclaration).orElse(null)));
+        }
+        if (!isImplicitImport(importDeclaration) && getImports().stream().noneMatch(im -> im.equals(importDeclaration) || (im.isAsterisk() && Objects.equals(getImportPackageName(im).get(), getImportPackageName(importDeclaration).orElse(null))))) {
             getImports().add(importDeclaration);
         }
         return this;
     }
 
+    /**
+     * @param importDeclaration
+     * @return {@code true}, if the import is implicit
+     */
+    private boolean isImplicitImport(ImportDeclaration importDeclaration) {
+        Optional<Name> importPackageName = getImportPackageName(importDeclaration);
+        if (importPackageName.isPresent()) {
+            if (parseName(JAVA_LANG).equals(importPackageName.get())) {
+                // java.lang is implicitly imported
+                return true;
+            }
+            if (packageDeclaration != null) {
+                // the import is within the same package
+                Name currentPackageName = packageDeclaration.getName();
+                return currentPackageName.equals(importPackageName.get());
+            }
+            return false;
+        }
+        return true;
+    }
+
+    private static Optional<Name> getImportPackageName(ImportDeclaration importDeclaration) {
+        return (importDeclaration.isAsterisk() ? new Name(importDeclaration.getName(), "*") : importDeclaration.getName()).getQualifier();
+    }
+
     /**
      * Sets or clear the package declarations of this compilation unit.
      *
-     * @param packageDeclaration the packageDeclaration declaration to set or <code>null</code> to default package
+     * @param packageDeclaration the packageDeclaration declaration to set or {@code null} to default package
      */
     @Generated("com.github.javaparser.generator.core.node.PropertyGenerator")
     public CompilationUnit setPackageDeclaration(final PackageDeclaration packageDeclaration) {
         if (packageDeclaration == this.packageDeclaration) {
-            return (CompilationUnit) this;
+            return this;
         }
         notifyPropertyChange(ObservableProperty.PACKAGE_DECLARATION, this.packageDeclaration, packageDeclaration);
         if (this.packageDeclaration != null)
@@ -255,7 +335,7 @@ public class CompilationUnit extends Node {
     public CompilationUnit setTypes(final NodeList<TypeDeclaration<?>> types) {
         assertNotNull(types);
         if (types == this.types) {
-            return (CompilationUnit) this;
+            return this;
         }
         notifyPropertyChange(ObservableProperty.TYPES, this.types, types);
         if (this.types != null)
@@ -309,19 +389,17 @@ public class CompilationUnit extends Node {
      *
      * @param clazz the class to import
      * @return this, the {@link CompilationUnit}
-     * @throws RuntimeException if clazz is an anonymous or local class
+     * @throws IllegalArgumentException if clazz is an anonymous or local class
      */
     public CompilationUnit addImport(Class<?> clazz) {
         if (clazz.isArray()) {
             return addImport(clazz.getComponentType());
         }
-        if (ClassUtils.isPrimitiveOrWrapper(clazz) || "java.lang".equals(clazz.getPackage().getName()))
+        if (ClassUtils.isPrimitiveOrWrapper(clazz) || JAVA_LANG.equals(clazz.getPackage().getName()))
             return this;
-        else if (clazz.isMemberClass())
-            return addImport(clazz.getName().replace("$", "."));
-        else if (clazz.isAnonymousClass() || clazz.isLocalClass())
-            throw new RuntimeException(clazz.getName() + " is an anonymous or local class therefore it can't be added with addImport");
-        return addImport(clazz.getName());
+        if (clazz.isAnonymousClass() || clazz.isLocalClass())
+            throw new IllegalArgumentException(clazz.getName() + " is an anonymous or local class therefore it can't be added with addImport");
+        return addImport(clazz.getCanonicalName());
     }
 
     /**
@@ -334,6 +412,9 @@ public class CompilationUnit extends Node {
      * @return this, the {@link CompilationUnit}
      */
     public CompilationUnit addImport(String name, boolean isStatic, boolean isAsterisk) {
+        if (name == null) {
+            return this;
+        }
         final StringBuilder i = new StringBuilder("import ");
         if (isStatic) {
             i.append("static ");
@@ -447,6 +528,15 @@ public class CompilationUnit extends Node {
         return getTypes().stream().filter(type -> type.getNameAsString().equals(className) && type instanceof ClassOrInterfaceDeclaration && !((ClassOrInterfaceDeclaration) type).isInterface()).findFirst().map(t -> (ClassOrInterfaceDeclaration) t);
     }
 
+    /**
+     * Try to get all local class declarations ending by its name (top level or inner class)
+     *
+     * @param className the class name (case-sensitive)
+     */
+    public List<ClassOrInterfaceDeclaration> getLocalDeclarationFromClassname(String className) {
+        return findAll(ClassOrInterfaceDeclaration.class).stream().filter(cid -> cid.getFullyQualifiedName().get().endsWith(className)).collect(Collectors.toList());
+    }
+
     /**
      * Try to get a top level interface declaration by its name
      *
@@ -491,11 +581,21 @@ public class CompilationUnit extends Node {
         return getTypes().stream().filter(type -> type.getNameAsString().equals(annotationName) && type instanceof AnnotationDeclaration).findFirst().map(t -> (AnnotationDeclaration) t);
     }
 
+    /**
+     * Try to get a top level record declaration by its name
+     *
+     * @param recordName the enum name (case-sensitive)
+     */
+    public Optional<RecordDeclaration> getRecordByName(String recordName) {
+        return getTypes().stream().filter(type -> type.getNameAsString().equals(recordName) && type instanceof RecordDeclaration).findFirst().map(t -> (RecordDeclaration) t);
+    }
+
     @Override
     @Generated("com.github.javaparser.generator.core.node.RemoveMethodGenerator")
     public boolean remove(Node node) {
-        if (node == null)
+        if (node == null) {
             return false;
+        }
         for (int i = 0; i < imports.size(); i++) {
             if (imports.get(i) == node) {
                 imports.remove(i);
@@ -539,7 +639,7 @@ public class CompilationUnit extends Node {
     @Generated("com.github.javaparser.generator.core.node.PropertyGenerator")
     public CompilationUnit setModule(final ModuleDeclaration module) {
         if (module == this.module) {
-            return (CompilationUnit) this;
+            return this;
         }
         notifyPropertyChange(ObservableProperty.MODULE, this.module, module);
         if (this.module != null)
@@ -566,6 +666,11 @@ public class CompilationUnit extends Node {
         return this;
     }
 
+    public CompilationUnit setStorage(Path path, Charset charset) {
+        this.storage = new Storage(this, path, charset);
+        return this;
+    }
+
     /**
      * Create (or overwrite) a module declaration in this compilation unit with name "name".
      *
@@ -608,9 +713,16 @@ public class CompilationUnit extends Node {
 
         private final Path path;
 
+        private final Charset encoding;
+
         private Storage(CompilationUnit compilationUnit, Path path) {
+            this(compilationUnit, path, UTF8);
+        }
+
+        private Storage(CompilationUnit compilationUnit, Path path, Charset encoding) {
             this.compilationUnit = compilationUnit;
             this.path = path.toAbsolutePath();
+            this.encoding = encoding;
         }
 
         /**
@@ -627,6 +739,13 @@ public class CompilationUnit extends Node {
             return compilationUnit;
         }
 
+        /**
+         * @return the encoding used to read the file.
+         */
+        public Charset getEncoding() {
+            return encoding;
+        }
+
         /**
          * @return the source root directory, calculated from the path of this compiation unit, and the package
          * declaration of this compilation unit. If the package declaration is invalid (when it does not match the end
@@ -634,7 +753,7 @@ public class CompilationUnit extends Node {
          */
         public Path getSourceRoot() {
             final Optional<String> pkgAsString = compilationUnit.getPackageDeclaration().map(NodeWithName::getNameAsString);
-            return pkgAsString.map(p -> Paths.get(CodeGenerationUtils.packageToPath(p))).map(pkg -> subtractPaths(getDirectory(), pkg)).orElse(getDirectory());
+            return pkgAsString.map(p -> Paths.get(CodeGenerationUtils.packageToPath(p))).map(pkg -> subtractPaths(getDirectory(), pkg)).orElseGet(() -> getDirectory());
         }
 
         public String getFileName() {
@@ -649,20 +768,31 @@ public class CompilationUnit extends Node {
          * Saves the compilation unit to its original location
          */
         public void save() {
-            save(cu -> new PrettyPrinter().print(cu));
+            save(cu -> compilationUnit.getPrinter().print(cu));
         }
 
         /**
-         * Saves a compilation unit to its original location with formatting according to the function
-         * passed as a parameter.
+         * Saves a compilation unit to its original location with formatting according to the function passed as a
+         * parameter.
          *
          * @param makeOutput a function that formats the compilation unit
          */
         public void save(Function<CompilationUnit, String> makeOutput) {
+            save(makeOutput, encoding);
+        }
+
+        /**
+         * Saves a compilation unit to its original location with formatting and encoding according to the function and
+         * encoding passed as a parameter.
+         *
+         * @param makeOutput a function that formats the compilation unit
+         * @param encoding the encoding to use for the saved file
+         */
+        public void save(Function<CompilationUnit, String> makeOutput, Charset encoding) {
             try {
                 Files.createDirectories(path.getParent());
                 final String code = makeOutput.apply(getCompilationUnit());
-                Files.write(path, code.getBytes(UTF8));
+                Files.write(path, code.getBytes(encoding));
             } catch (IOException e) {
                 throw new RuntimeException(e);
             }
@@ -692,8 +822,9 @@ public class CompilationUnit extends Node {
     @Override
     @Generated("com.github.javaparser.generator.core.node.ReplaceMethodGenerator")
     public boolean replace(Node node, Node replacementNode) {
-        if (node == null)
+        if (node == null) {
             return false;
+        }
         for (int i = 0; i < imports.size(); i++) {
             if (imports.get(i) == node) {
                 imports.set(i, (ImportDeclaration) replacementNode);
