@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -20,13 +20,30 @@
  */
 package com.github.javaparser.ast;
 
+import static com.github.javaparser.ast.Node.Parsedness.PARSED;
+import static com.github.javaparser.ast.Node.TreeTraversal.PREORDER;
+import static java.util.Collections.emptySet;
+import static java.util.Collections.unmodifiableList;
+import static java.util.Spliterator.DISTINCT;
+import static java.util.Spliterator.NONNULL;
+
+import java.util.*;
+import java.util.function.Consumer;
+import java.util.function.Function;
+import java.util.function.Predicate;
+import java.util.stream.Stream;
+import java.util.stream.StreamSupport;
+
 import com.github.javaparser.HasParentNode;
+import com.github.javaparser.Position;
 import com.github.javaparser.Range;
 import com.github.javaparser.TokenRange;
 import com.github.javaparser.ast.comments.BlockComment;
 import com.github.javaparser.ast.comments.Comment;
 import com.github.javaparser.ast.comments.LineComment;
+import com.github.javaparser.ast.nodeTypes.NodeWithOptionalScope;
 import com.github.javaparser.ast.nodeTypes.NodeWithRange;
+import com.github.javaparser.ast.nodeTypes.NodeWithScope;
 import com.github.javaparser.ast.nodeTypes.NodeWithTokenRange;
 import com.github.javaparser.ast.observer.AstObserver;
 import com.github.javaparser.ast.observer.ObservableProperty;
@@ -36,25 +53,14 @@ import com.github.javaparser.ast.visitor.EqualsVisitor;
 import com.github.javaparser.ast.visitor.HashCodeVisitor;
 import com.github.javaparser.ast.visitor.Visitable;
 import com.github.javaparser.metamodel.*;
-import com.github.javaparser.printer.PrettyPrinter;
-import com.github.javaparser.printer.PrettyPrinterConfiguration;
+import com.github.javaparser.printer.DefaultPrettyPrinter;
+import com.github.javaparser.printer.Printer;
+import com.github.javaparser.printer.configuration.DefaultConfigurationOption;
+import com.github.javaparser.printer.configuration.DefaultPrinterConfiguration;
+import com.github.javaparser.printer.configuration.DefaultPrinterConfiguration.ConfigOption;
+import com.github.javaparser.printer.configuration.PrinterConfiguration;
 import com.github.javaparser.resolution.SymbolResolver;
-import com.github.javaparser.resolution.types.ResolvedType;
-import java.util.*;
-import java.util.function.Consumer;
-import java.util.function.Function;
-import java.util.function.Predicate;
-import java.util.stream.Stream;
-import java.util.stream.StreamSupport;
-import static com.github.javaparser.ast.Node.Parsedness.PARSED;
-import static com.github.javaparser.ast.Node.TreeTraversal.PREORDER;
-import static java.util.Collections.emptySet;
-import static java.util.Collections.unmodifiableList;
-import static java.util.Spliterator.DISTINCT;
-import static java.util.Spliterator.NONNULL;
-import com.github.javaparser.metamodel.NodeMetaModel;
-import com.github.javaparser.metamodel.JavaParserMetaModel;
-import com.github.javaparser.ast.Node;
+import com.github.javaparser.utils.LineSeparator;
 
 /**
  * Base class for all nodes of the abstract syntax tree.
@@ -88,7 +94,7 @@ import com.github.javaparser.ast.Node;
  * like where braces or comma's are exactly.
  * Therefore there is no position information on everything in the original source file.
  * <h2>Observers</h2>
- * <p>It is possible to add observers to the the tree.
+ * <p>It is possible to add observers to the tree.
  * Any change in the tree is sent as an event to any observers watching.
  * <h2>Visitors</h2>
  * <p>The most comfortable way of working with an abstract syntax tree is using visitors.
@@ -132,11 +138,11 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
      * This can be used to sort nodes on position.
      */
     public static Comparator<NodeWithRange<?>> NODE_BY_BEGIN_POSITION = (a, b) -> {
-        if (a.getRange().isPresent() && b.getRange().isPresent()) {
+        if (a.hasRange() && b.hasRange()) {
             return a.getRange().get().begin.compareTo(b.getRange().get().begin);
         }
-        if (a.getRange().isPresent() || b.getRange().isPresent()) {
-            if (a.getRange().isPresent()) {
+        if (a.hasRange() || b.hasRange()) {
+            if (a.hasRange()) {
                 return 1;
             }
             return -1;
@@ -144,9 +150,11 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         return 0;
     };
 
-    private static PrettyPrinterConfiguration toStringPrettyPrinterConfiguration = new PrettyPrinterConfiguration();
+    // usefull to find if the node is a phantom node
+    private static final int LEVELS_TO_EXPLORE = 3;
 
-    protected static final PrettyPrinterConfiguration prettyPrinterNoCommentsConfiguration = new PrettyPrinterConfiguration().setPrintComments(false);
+    protected static final PrinterConfiguration prettyPrinterNoCommentsConfiguration = new DefaultPrinterConfiguration()
+			.removeOption(new DefaultConfigurationOption(ConfigOption.PRINT_COMMENTS));
 
     @InternalProperty
     private Range range;
@@ -158,10 +166,10 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
     private Node parentNode;
 
     @InternalProperty
-    private List<Node> childNodes = new LinkedList<>();
+    private ArrayList<Node> childNodes = new ArrayList<>(0);
 
     @InternalProperty
-    private List<Comment> orphanComments = new LinkedList<>();
+    private ArrayList<Comment> orphanComments = new ArrayList<>(0);
 
     @InternalProperty
     private IdentityHashMap<DataKey<?>, Object> data = null;
@@ -170,7 +178,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
     private Comment comment;
 
     @InternalProperty
-    private List<AstObserver> observers = new ArrayList<>();
+    private ArrayList<AstObserver> observers = new ArrayList<>(0);
 
     @InternalProperty
     private Parsedness parsed = PARSED;
@@ -187,6 +195,36 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
     protected void customInitialization() {
     }
 
+    /*
+     * If there is a printer defined in CompilationUnit, returns it
+     * else create a new DefaultPrettyPrinter with default parameters
+     */
+    protected Printer getPrinter() {
+        return findCompilationUnit().map(c -> c.getPrinter()).orElseGet(() -> createDefaultPrinter());
+    }
+
+    /*
+     * Return the printer initialized with the specified configuration
+     */
+    protected Printer getPrinter(PrinterConfiguration configuration) {
+        return findCompilationUnit().map(c -> c.getPrinter(configuration)).orElseGet(() -> createDefaultPrinter(configuration));
+    }
+
+    protected Printer createDefaultPrinter() {
+        return createDefaultPrinter(getDefaultPrinterConfiguration());
+    }
+
+    protected Printer createDefaultPrinter(PrinterConfiguration configuration) {
+        return new DefaultPrettyPrinter(configuration);
+    }
+
+    /*
+     * returns a default printer configuration
+     */
+    protected PrinterConfiguration getDefaultPrinterConfiguration() {
+        return new DefaultPrinterConfiguration();
+    }
+
     /**
      * This is a comment associated with this node.
      *
@@ -200,20 +238,23 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
     /**
      * @return the range of characters in the source code that this node covers.
      */
-    public Optional<Range> getRange() {
+    @Override
+	public Optional<Range> getRange() {
         return Optional.ofNullable(range);
     }
 
     /**
      * @return the range of tokens that this node covers.
      */
-    public Optional<TokenRange> getTokenRange() {
+    @Override
+	public Optional<TokenRange> getTokenRange() {
         return Optional.ofNullable(tokenRange);
     }
 
-    public Node setTokenRange(TokenRange tokenRange) {
+    @Override
+	public Node setTokenRange(TokenRange tokenRange) {
         this.tokenRange = tokenRange;
-        if (tokenRange == null || !(tokenRange.getBegin().getRange().isPresent() && tokenRange.getBegin().getRange().isPresent())) {
+        if (tokenRange == null || !(tokenRange.getBegin().hasRange() && tokenRange.getEnd().hasRange())) {
             range = null;
         } else {
             range = new Range(tokenRange.getBegin().getRange().get().begin, tokenRange.getEnd().getRange().get().end);
@@ -223,9 +264,10 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
 
     /**
      * @param range the range of characters in the source code that this node covers. null can be used to indicate that
-     * no range information is known, or that it is not of interest.
+     *              no range information is known, or that it is not of interest.
      */
-    public Node setRange(Range range) {
+    @Override
+	public Node setRange(Range range) {
         if (this.range == range) {
             return this;
         }
@@ -239,13 +281,10 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
      *
      * @param comment to be set
      */
-    public final Node setComment(final Comment comment) {
+    public Node setComment(final Comment comment) {
         if (this.comment == comment) {
             return this;
         }
-        if (comment != null && (this instanceof Comment)) {
-            throw new RuntimeException("A comment can not be commented");
-        }
         notifyPropertyChange(ObservableProperty.COMMENT, this.comment, comment);
         if (this.comment != null) {
             this.comment.setCommentedNode(null);
@@ -277,19 +316,32 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
 
     /**
      * @return pretty printed source code for this node and its children.
-     * Formatting can be configured with Node.setToStringPrettyPrinterConfiguration.
      */
     @Override
     public final String toString() {
-        return new PrettyPrinter(toStringPrettyPrinterConfiguration).print(this);
+        if (containsData(LINE_SEPARATOR_KEY)) {
+            Printer printer = getPrinter();
+            LineSeparator lineSeparator = getLineEndingStyleOrDefault(LineSeparator.SYSTEM);
+            PrinterConfiguration config = printer.getConfiguration();
+            config.addOption(new DefaultConfigurationOption(ConfigOption.END_OF_LINE_CHARACTER, lineSeparator.asRawString()));
+            printer.setConfiguration(config);
+            return printer.print(this);
+        }
+        return getPrinter().print(this);
     }
 
     /**
      * @return pretty printed source code for this node and its children.
-     * Formatting can be configured with parameter prettyPrinterConfiguration.
+     * Formatting can be configured with parameter PrinterConfiguration.
      */
-    public final String toString(PrettyPrinterConfiguration prettyPrinterConfiguration) {
-        return new PrettyPrinter(prettyPrinterConfiguration).print(this);
+    public final String toString(PrinterConfiguration configuration) {
+    	// save the current configuration
+    	PrinterConfiguration previousConfiguration = getPrinter().getConfiguration();
+    	// print with the new configuration
+    	String result = getPrinter(configuration).print(this);
+    	// restore the previous printer configuration (issue 4163)
+    	getPrinter().setConfiguration(previousConfiguration);
+    	return result;
     }
 
     @Override
@@ -299,7 +351,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
 
     @Override
     public boolean equals(final Object obj) {
-        if (obj == null || !(obj instanceof Node)) {
+        if (!(obj instanceof Node)) {
             return false;
         }
         return EqualsVisitor.equals(this, (Node) obj);
@@ -321,6 +373,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
     }
 
     public void addOrphanComment(Comment comment) {
+    	notifyPropertyChange(ObservableProperty.COMMENT, null, comment);
         orphanComments.add(comment);
         comment.setParentNode(this);
     }
@@ -330,6 +383,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         if (removed) {
             notifyPropertyChange(ObservableProperty.COMMENT, comment, null);
             comment.setParentNode(null);
+            orphanComments.trimToSize();
         }
         return removed;
     }
@@ -349,7 +403,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
      * @return all comments that cannot be attributed to a concept
      */
     public List<Comment> getOrphanComments() {
-        return new LinkedList<>(orphanComments);
+        return unmodifiableList(orphanComments);
     }
 
     /**
@@ -360,8 +414,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
      * @return all Comments within the node as a list
      */
     public List<Comment> getAllContainedComments() {
-        List<Comment> comments = new LinkedList<>();
-        comments.addAll(getOrphanComments());
+        List<Comment> comments = new LinkedList<>(orphanComments);
         for (Node child : getChildNodes()) {
             child.getComment().ifPresent(comments::add);
             comments.addAll(child.getAllContainedComments());
@@ -383,12 +436,13 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         observers.forEach(o -> o.parentChange(this, parentNode, newParentNode));
         // remove from old parent, if any
         if (parentNode != null) {
-            final List<Node> parentChildNodes = parentNode.childNodes;
+            final ArrayList<Node> parentChildNodes = parentNode.childNodes;
             for (int i = 0; i < parentChildNodes.size(); i++) {
                 if (parentChildNodes.get(i) == this) {
                     parentChildNodes.remove(i);
                 }
             }
+            parentChildNodes.trimToSize();
         }
         parentNode = newParentNode;
         // add to new parent, if any
@@ -404,9 +458,17 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         }
     }
 
-    public static final int ABSOLUTE_BEGIN_LINE = -1;
+    /**
+     * @deprecated Use {@link Position#ABSOLUTE_BEGIN_LINE}
+     */
+    @Deprecated
+    public static final int ABSOLUTE_BEGIN_LINE = Position.ABSOLUTE_BEGIN_LINE;
 
-    public static final int ABSOLUTE_END_LINE = -2;
+    /**
+     * @deprecated Use {@link Position#ABSOLUTE_END_LINE}
+     */
+    @Deprecated
+    public static final int ABSOLUTE_END_LINE = Position.ABSOLUTE_END_LINE;
 
     public void tryAddImportToParentCompilationUnit(Class<?> clazz) {
         findAncestor(CompilationUnit.class).ifPresent(p -> p.addImport(clazz));
@@ -460,6 +522,22 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         return value;
     }
 
+    /**
+     * Gets data for this node using the given key or returns an {@code Optional.empty()}.
+     *
+     * @param <M> The type of the data.
+     * @param key The key for the data
+     * @return The data.
+     * @see DataKey
+     */
+    @SuppressWarnings("unchecked")
+    public <M> Optional<M> findData(final DataKey<M> key) {
+    	if (containsData(key)) {
+            return Optional.of(getData(key));
+        }
+        return Optional.empty();
+    }
+
     /**
      * This method was added to support the clone method.
      *
@@ -476,8 +554,8 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
      * Sets data for this node using the given key.
      * For information on creating DataKey, see {@link DataKey}.
      *
-     * @param <M> The type of data
-     * @param key The singleton key for the data
+     * @param <M>    The type of data
+     * @param key    The singleton key for the data
      * @param object The data object
      * @see DataKey
      */
@@ -504,7 +582,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
      *
      * @see DataKey
      */
-    public void removeData(DataKey<ResolvedType> key) {
+    public void removeData(DataKey<?> key) {
         if (data != null) {
             data.remove(key);
         }
@@ -570,11 +648,16 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
     @Override
     public void unregister(AstObserver observer) {
         this.observers.remove(observer);
+        this.observers.trimToSize();
     }
 
     @Override
     public void register(AstObserver observer) {
-        this.observers.add(observer);
+        // Check if the observer is not registered yet.
+        // In this case we use a List instead of Set to save on memory space.
+        if (!this.observers.contains(observer)) {
+            this.observers.add(observer);
+        }
     }
 
     /**
@@ -622,8 +705,9 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
 
     @Generated("com.github.javaparser.generator.core.node.RemoveMethodGenerator")
     public boolean remove(Node node) {
-        if (node == null)
+        if (node == null) {
             return false;
+        }
         if (comment != null) {
             if (node == comment) {
                 removeComment();
@@ -668,18 +752,11 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         return this;
     }
 
-    public static PrettyPrinterConfiguration getToStringPrettyPrinterConfiguration() {
-        return toStringPrettyPrinterConfiguration;
-    }
-
-    public static void setToStringPrettyPrinterConfiguration(PrettyPrinterConfiguration toStringPrettyPrinterConfiguration) {
-        Node.toStringPrettyPrinterConfiguration = toStringPrettyPrinterConfiguration;
-    }
-
     @Generated("com.github.javaparser.generator.core.node.ReplaceMethodGenerator")
     public boolean replace(Node node, Node replacementNode) {
-        if (node == null)
+        if (node == null) {
             return false;
+        }
         if (comment != null) {
             if (node == comment) {
                 setComment((Comment) replacementNode);
@@ -711,13 +788,37 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         return Optional.empty();
     }
 
-    protected SymbolResolver getSymbolResolver() {
-        return findCompilationUnit().map(cu -> {
-            SymbolResolver symbolResolver = cu.getData(SYMBOL_RESOLVER_KEY);
-            if (symbolResolver == null) {
-                throw new IllegalStateException("Symbol resolution not configured: to configure consider setting a SymbolResolver in the ParserConfiguration");
+    public LineSeparator getLineEndingStyleOrDefault(LineSeparator defaultLineSeparator) {
+        if (getLineEndingStyle().isStandardEol()) {
+            return getLineEndingStyle();
+        }
+        return defaultLineSeparator;
+    }
+
+    public LineSeparator getLineEndingStyle() {
+        Node current = this;
+        // First check this node
+        if (current.containsData(Node.LINE_SEPARATOR_KEY)) {
+            LineSeparator lineSeparator = current.getData(Node.LINE_SEPARATOR_KEY);
+            return lineSeparator;
+        }
+        // Then check parent/ancestor nodes
+        while (current.getParentNode().isPresent()) {
+            current = current.getParentNode().get();
+            if (current.containsData(Node.LINE_SEPARATOR_KEY)) {
+                return current.getData(Node.LINE_SEPARATOR_KEY);
             }
-            return symbolResolver;
+        }
+        // Default to the system line separator if it's not already set within the parsed node/code.
+        return LineSeparator.SYSTEM;
+    }
+
+    public SymbolResolver getSymbolResolver() {
+        return findCompilationUnit().map(cu -> {
+            if (cu.containsData(SYMBOL_RESOLVER_KEY)) {
+                return cu.getData(SYMBOL_RESOLVER_KEY);
+            }
+            throw new IllegalStateException("Symbol resolution not configured: to configure consider setting a SymbolResolver in the ParserConfiguration");
         }).orElseThrow(() -> new IllegalStateException("The node is not inserted in a CompilationUnit"));
     }
 
@@ -725,6 +826,15 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
     public static final DataKey<SymbolResolver> SYMBOL_RESOLVER_KEY = new DataKey<SymbolResolver>() {
     };
 
+    public static final DataKey<LineSeparator> LINE_SEPARATOR_KEY = new DataKey<LineSeparator>() {
+    };
+
+    protected static final DataKey<Printer> PRINTER_KEY = new DataKey<Printer>() {
+    };
+
+    protected static final DataKey<Boolean> PHANTOM_KEY = new DataKey<Boolean>() {
+    };
+
     public enum TreeTraversal {
 
         PREORDER, BREADTHFIRST, POSTORDER, PARENTS, DIRECT_CHILDREN
@@ -767,7 +877,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
 
     /**
      * Walks the AST, calling the consumer for every node, with traversal algorithm "traversal".
-     * <br/>This is the most general walk method. All other walk and findAll methods are based on this.
+     * <br>This is the most general walk method. All other walk and findAll methods are based on this.
      */
     public void walk(TreeTraversal traversal, Consumer<Node> consumer) {
         // Could be implemented as a call to the above walk method, but this is a little more efficient.
@@ -803,6 +913,19 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         return found;
     }
 
+    /**
+     * Walks the AST with specified traversal order, returning all nodes of type "nodeType".
+     */
+    public <T extends Node> List<T> findAll(Class<T> nodeType, TreeTraversal traversal) {
+        final List<T> found = new ArrayList<>();
+        walk(traversal, node -> {
+            if (nodeType.isAssignableFrom(node.getClass())) {
+                found.add(nodeType.cast(node));
+            }
+        });
+        return found;
+    }
+
     /**
      * Walks the AST with pre-order traversal, returning all nodes of type "nodeType" that match the predicate.
      */
@@ -817,7 +940,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
 
     /**
      * Walks the AST, applying the function for every node, with traversal algorithm "traversal". If the function
-     * returns something else than null, the traversal is stopped and the function result is returned. <br/>This is the
+     * returns something else than null, the traversal is stopped and the function result is returned. <br>This is the
      * most general findFirst method. All other findFirst methods are based on this.
      */
     public <T> Optional<T> findFirst(TreeTraversal traversal, Function<Node, Optional<T>> consumer) {
@@ -858,6 +981,17 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         });
     }
 
+    /**
+     * Determines whether this node is an ancestor of the given node. A node is <i>not</i> an ancestor of itself.
+     *
+     * @param descendant the node for which to determine whether it has this node as an ancestor.
+     * @return {@code true} if this node is an ancestor of the given node, and {@code false} otherwise.
+     * @see HasParentNode#isDescendantOf(Node)
+     */
+    public boolean isAncestorOf(Node descendant) {
+        return this != descendant && findFirst(Node.class, n -> n == descendant).isPresent();
+    }
+
     /**
      * Performs a breadth-first node traversal starting with a given node.
      *
@@ -892,7 +1026,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
         private final Iterator<Node> childrenIterator;
 
         public DirectChildrenIterator(Node node) {
-            childrenIterator = new ArrayList<>(node.getChildNodes()).iterator();
+            childrenIterator = node.getChildNodes().iterator();
         }
 
         @Override
@@ -981,7 +1115,7 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
 
         private void fillStackToLeaf(Node node) {
             while (true) {
-                List<Node> childNodes = new ArrayList<>(node.getChildNodes());
+                List<Node> childNodes = node.getChildNodes();
                 if (childNodes.isEmpty()) {
                     break;
                 }
@@ -1005,15 +1139,14 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
                 Node node = nodes.get(cursor);
                 fillStackToLeaf(node);
                 return nextFromLevel();
-            } else {
-                nodesStack.pop();
-                cursorStack.pop();
-                hasNext = !nodesStack.empty();
-                if (hasNext) {
+            }
+            nodesStack.pop();
+            cursorStack.pop();
+            hasNext = !nodesStack.empty();
+            if (hasNext) {
                     return nextFromLevel();
                 }
-                return root;
-            }
+            return root;
         }
 
         private Node nextFromLevel() {
@@ -1023,4 +1156,34 @@ public abstract class Node implements Cloneable, HasParentNode<Node>, Visitable,
             return nodes.get(cursor);
         }
     }
+
+    /*
+     * Returns true if the node has an (optional) scope expression eg. method calls (object.method())
+     */
+    public boolean hasScope() {
+        return (NodeWithOptionalScope.class.isAssignableFrom(this.getClass()) && ((NodeWithOptionalScope) this).getScope().isPresent())
+        		|| (NodeWithScope.class.isAssignableFrom(this.getClass()) && ((NodeWithScope) this).getScope() != null);
+    }
+
+    /*
+     * A "phantom" node, is a node that is not really an AST node (like the fake type of variable in FieldDeclaration or an UnknownType)
+     */
+    public boolean isPhantom() {
+        return isPhantom(this);
+    }
+
+    private boolean isPhantom(Node node) {
+        if (!node.containsData(PHANTOM_KEY)) {
+            boolean res = (node.getParentNode().isPresent() && node.getParentNode().get().hasRange() && node.hasRange() && !node.getParentNode().get().getRange().get().contains(node.getRange().get()) || inPhantomNode(node, LEVELS_TO_EXPLORE));
+            node.setData(PHANTOM_KEY, res);
+        }
+        return node.getData(PHANTOM_KEY);
+    }
+
+    /**
+     * A node contained in a phantom node is also a phantom node. We limit how many levels up we check just for performance reasons.
+     */
+    private boolean inPhantomNode(Node node, int levels) {
+        return node.getParentNode().isPresent() && (isPhantom(node.getParentNode().get()) || inPhantomNode(node.getParentNode().get(), levels - 1));
+    }
 }
