@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -262,8 +262,8 @@ public abstract class GenericVisitorAdapter<R, A> implements GenericVisitor<R, A
     @Override
     public R visit(final BreakStmt n, final A arg) {
         R result;
-        if (n.getValue().isPresent()) {
-            result = n.getValue().get().accept(this, arg);
+        if (n.getLabel().isPresent()) {
+            result = n.getLabel().get().accept(this, arg);
             if (result != null)
                 return result;
         }
@@ -357,6 +357,11 @@ public abstract class GenericVisitorAdapter<R, A> implements GenericVisitor<R, A
             if (result != null)
                 return result;
         }
+        {
+            result = n.getPermittedTypes().accept(this, arg);
+            if (result != null)
+                return result;
+        }
         {
             result = n.getTypeParameters().accept(this, arg);
             if (result != null)
@@ -882,6 +887,11 @@ public abstract class GenericVisitorAdapter<R, A> implements GenericVisitor<R, A
             if (result != null)
                 return result;
         }
+        if (n.getPattern().isPresent()) {
+            result = n.getPattern().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
         {
             result = n.getType().accept(this, arg);
             if (result != null)
@@ -1392,8 +1402,8 @@ public abstract class GenericVisitorAdapter<R, A> implements GenericVisitor<R, A
     @Override
     public R visit(final SuperExpr n, final A arg) {
         R result;
-        if (n.getClassExpr().isPresent()) {
-            result = n.getClassExpr().get().accept(this, arg);
+        if (n.getTypeName().isPresent()) {
+            result = n.getTypeName().get().accept(this, arg);
             if (result != null)
                 return result;
         }
@@ -1471,8 +1481,8 @@ public abstract class GenericVisitorAdapter<R, A> implements GenericVisitor<R, A
     @Override
     public R visit(final ThisExpr n, final A arg) {
         R result;
-        if (n.getClassExpr().isPresent()) {
-            result = n.getClassExpr().get().accept(this, arg);
+        if (n.getTypeName().isPresent()) {
+            result = n.getTypeName().get().accept(this, arg);
             if (result != null)
                 return result;
         }
@@ -1547,6 +1557,22 @@ public abstract class GenericVisitorAdapter<R, A> implements GenericVisitor<R, A
         return null;
     }
 
+    @Override
+    public R visit(final LocalRecordDeclarationStmt n, final A arg) {
+        R result;
+        {
+            result = n.getRecordDeclaration().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        if (n.getComment().isPresent()) {
+            result = n.getComment().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        return null;
+    }
+
     @Override
     public R visit(final TypeParameter n, final A arg) {
         R result;
@@ -2037,4 +2063,149 @@ public abstract class GenericVisitorAdapter<R, A> implements GenericVisitor<R, A
         }
         return null;
     }
+
+    @Override
+    public R visit(final YieldStmt n, final A arg) {
+        R result;
+        {
+            result = n.getExpression().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        if (n.getComment().isPresent()) {
+            result = n.getComment().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        return null;
+    }
+
+    @Override
+    public R visit(final TextBlockLiteralExpr n, final A arg) {
+        R result;
+        if (n.getComment().isPresent()) {
+            result = n.getComment().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        return null;
+    }
+
+    @Override
+    public R visit(final PatternExpr n, final A arg) {
+        R result;
+        {
+            result = n.getModifiers().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getName().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getType().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        if (n.getComment().isPresent()) {
+            result = n.getComment().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        return null;
+    }
+
+    @Override
+    public R visit(final RecordDeclaration n, final A arg) {
+        R result;
+        {
+            result = n.getImplementedTypes().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getParameters().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        if (n.getReceiverParameter().isPresent()) {
+            result = n.getReceiverParameter().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getTypeParameters().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getMembers().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getModifiers().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getName().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getAnnotations().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        if (n.getComment().isPresent()) {
+            result = n.getComment().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        return null;
+    }
+
+    @Override
+    public R visit(final CompactConstructorDeclaration n, final A arg) {
+        R result;
+        {
+            result = n.getBody().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getModifiers().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getName().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getThrownExceptions().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getTypeParameters().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        {
+            result = n.getAnnotations().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        if (n.getComment().isPresent()) {
+            result = n.getComment().get().accept(this, arg);
+            if (result != null)
+                return result;
+        }
+        return null;
+    }
 }
