@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -29,6 +29,7 @@ import com.github.javaparser.ast.expr.*;
 import com.github.javaparser.ast.modules.*;
 import com.github.javaparser.ast.stmt.*;
 import com.github.javaparser.ast.type.*;
+
 import java.util.Optional;
 
 public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable> {
@@ -143,6 +144,8 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
             return false;
         if (!objEquals(n.isInterface(), n2.isInterface()))
             return false;
+        if (!nodesEquals(n.getPermittedTypes(), n2.getPermittedTypes()))
+            return false;
         if (!nodesEquals(n.getTypeParameters(), n2.getTypeParameters()))
             return false;
         if (!nodesEquals(n.getMembers(), n2.getMembers()))
@@ -526,6 +529,8 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
         final InstanceOfExpr n2 = (InstanceOfExpr) arg;
         if (!nodeEquals(n.getExpression(), n2.getExpression()))
             return false;
+        if (!nodeEquals(n.getPattern(), n2.getPattern()))
+            return false;
         if (!nodeEquals(n.getType(), n2.getType()))
             return false;
         return true;
@@ -574,7 +579,7 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
     @Override
     public Boolean visit(final BooleanLiteralExpr n, final Visitable arg) {
         final BooleanLiteralExpr n2 = (BooleanLiteralExpr) arg;
-        if (!objEquals(n.getValue(), n2.getValue()))
+        if (!objEquals(n.isValue(), n2.isValue()))
             return false;
         return true;
     }
@@ -643,7 +648,7 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
     @Override
     public Boolean visit(final ThisExpr n, final Visitable arg) {
         final ThisExpr n2 = (ThisExpr) arg;
-        if (!nodeEquals(n.getClassExpr(), n2.getClassExpr()))
+        if (!nodeEquals(n.getTypeName(), n2.getTypeName()))
             return false;
         return true;
     }
@@ -651,7 +656,7 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
     @Override
     public Boolean visit(final SuperExpr n, final Visitable arg) {
         final SuperExpr n2 = (SuperExpr) arg;
-        if (!nodeEquals(n.getClassExpr(), n2.getClassExpr()))
+        if (!nodeEquals(n.getTypeName(), n2.getTypeName()))
             return false;
         return true;
     }
@@ -738,6 +743,14 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
         return true;
     }
 
+    @Override
+    public Boolean visit(final LocalRecordDeclarationStmt n, final Visitable arg) {
+        final LocalRecordDeclarationStmt n2 = (LocalRecordDeclarationStmt) arg;
+        if (!nodeEquals(n.getRecordDeclaration(), n2.getRecordDeclaration()))
+            return false;
+        return true;
+    }
+
     @Override
     public Boolean visit(final AssertStmt n, final Visitable arg) {
         final AssertStmt n2 = (AssertStmt) arg;
@@ -804,7 +817,7 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
     @Override
     public Boolean visit(final BreakStmt n, final Visitable arg) {
         final BreakStmt n2 = (BreakStmt) arg;
-        if (!nodeEquals(n.getValue(), n2.getValue()))
+        if (!nodeEquals(n.getLabel(), n2.getLabel()))
             return false;
         return true;
     }
@@ -1078,4 +1091,72 @@ public class NoCommentEqualsVisitor implements GenericVisitor<Boolean, Visitable
             return false;
         return true;
     }
+
+    @Override
+    public Boolean visit(final YieldStmt n, final Visitable arg) {
+        final YieldStmt n2 = (YieldStmt) arg;
+        if (!nodeEquals(n.getExpression(), n2.getExpression()))
+            return false;
+        return true;
+    }
+
+    @Override
+    public Boolean visit(final TextBlockLiteralExpr n, final Visitable arg) {
+        final TextBlockLiteralExpr n2 = (TextBlockLiteralExpr) arg;
+        if (!objEquals(n.getValue(), n2.getValue()))
+            return false;
+        return true;
+    }
+
+    @Override
+    public Boolean visit(final PatternExpr n, final Visitable arg) {
+        final PatternExpr n2 = (PatternExpr) arg;
+        if (!nodesEquals(n.getModifiers(), n2.getModifiers()))
+            return false;
+        if (!nodeEquals(n.getName(), n2.getName()))
+            return false;
+        if (!nodeEquals(n.getType(), n2.getType()))
+            return false;
+        return true;
+    }
+
+    @Override
+    public Boolean visit(final RecordDeclaration n, final Visitable arg) {
+        final RecordDeclaration n2 = (RecordDeclaration) arg;
+        if (!nodesEquals(n.getImplementedTypes(), n2.getImplementedTypes()))
+            return false;
+        if (!nodesEquals(n.getParameters(), n2.getParameters()))
+            return false;
+        if (!nodeEquals(n.getReceiverParameter(), n2.getReceiverParameter()))
+            return false;
+        if (!nodesEquals(n.getTypeParameters(), n2.getTypeParameters()))
+            return false;
+        if (!nodesEquals(n.getMembers(), n2.getMembers()))
+            return false;
+        if (!nodesEquals(n.getModifiers(), n2.getModifiers()))
+            return false;
+        if (!nodeEquals(n.getName(), n2.getName()))
+            return false;
+        if (!nodesEquals(n.getAnnotations(), n2.getAnnotations()))
+            return false;
+        return true;
+    }
+
+    @Override
+    public Boolean visit(final CompactConstructorDeclaration n, final Visitable arg) {
+        final CompactConstructorDeclaration n2 = (CompactConstructorDeclaration) arg;
+        if (!nodeEquals(n.getBody(), n2.getBody()))
+            return false;
+        if (!nodesEquals(n.getModifiers(), n2.getModifiers()))
+            return false;
+        if (!nodeEquals(n.getName(), n2.getName()))
+            return false;
+        if (!nodesEquals(n.getThrownExceptions(), n2.getThrownExceptions()))
+            return false;
+        if (!nodesEquals(n.getTypeParameters(), n2.getTypeParameters()))
+            return false;
+        if (!nodesEquals(n.getAnnotations(), n2.getAnnotations()))
+            return false;
+        return true;
+    }
 }
