@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -18,40 +18,23 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  */
-
 package com.github.javaparser.printer.concretesyntaxmodel;
 
 import com.github.javaparser.ast.Node;
-import com.github.javaparser.ast.comments.BlockComment;
 import com.github.javaparser.ast.comments.Comment;
-import com.github.javaparser.ast.comments.JavadocComment;
-import com.github.javaparser.ast.comments.LineComment;
 import com.github.javaparser.printer.SourcePrinter;
 
 public class CsmComment implements CsmElement {
 
     static void process(Comment comment, SourcePrinter printer) {
         String content = printer.normalizeEolInTextBlock(comment.getContent());
-        if (comment instanceof BlockComment) {
-            printer.print("/*");
-            printer.print(content);
-            printer.println("*/");
-        } else if (comment instanceof JavadocComment) {
-            printer.print("/**");
-            printer.print(content);
-            printer.println("*/");
-        } else if (comment instanceof LineComment) {
-            printer.print("//");
-            printer.print(content);
-            printer.println();
-        } else {
-            throw new UnsupportedOperationException(comment.getClass().getSimpleName());
-        }
+		printer.print(comment.getHeader());
+		printer.print(content);
+		printer.println(comment.getFooter());
     }
 
     @Override
     public void prettyPrint(Node node, SourcePrinter printer) {
         node.getComment().ifPresent(c -> process(c, printer));
     }
-
 }
