@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
- * Copyright (C) 2011, 2013-2016 The JavaParser Team.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
  *
  * This file is part of JavaParser.
  *
@@ -18,18 +18,18 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  */
-
 package com.github.javaparser.resolution;
 
 import com.github.javaparser.ast.Node;
 import com.github.javaparser.ast.expr.Expression;
 import com.github.javaparser.ast.type.Type;
+import com.github.javaparser.resolution.declarations.ResolvedReferenceTypeDeclaration;
 import com.github.javaparser.resolution.types.ResolvedType;
 
 public interface SymbolResolver {
+
     /**
-     * For a reference it would find the corresponding
-     * declaration.
+     * For a reference it would find the corresponding declaration.
      */
     <T> T resolveDeclaration(Node node, Class<T> resultClass);
 
@@ -38,5 +38,13 @@ public interface SymbolResolver {
      */
     <T> T toResolvedType(Type javaparserType, Class<T> resultClass);
 
+    /**
+     * For an expression it would find the corresponding resolved type.
+     */
     ResolvedType calculateType(Expression expression);
+    
+    /**
+     * For a node it would find the corresponding reference type declaration.
+     */
+    ResolvedReferenceTypeDeclaration toTypeDeclaration(Node node);
 }
