@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2007-2010 Júlio Vilmar Gesser.
+ * Copyright (C) 2011, 2013-2024 The JavaParser Team.
+ *
+ * This file is part of JavaParser.
+ *
+ * JavaParser can be used either under the terms of
+ * a) the GNU Lesser General Public License as published by
+ *     the Free Software Foundation, either version 3 of the License, or
+ *     (at your option) any later version.
+ * b) the terms of the Apache License
+ *
+ * You should have received a copy of both licenses in LICENCE.LGPL and
+ * LICENCE.APACHE. Please refer to those files for details.
+ *
+ * JavaParser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ */
 package com.github.javaparser.utils;
 
 import com.github.javaparser.JavaParser;
@@ -5,9 +25,10 @@ import com.github.javaparser.ParseProblemException;
 import com.github.javaparser.ParseResult;
 import com.github.javaparser.ParserConfiguration;
 import com.github.javaparser.ast.CompilationUnit;
-import com.github.javaparser.printer.PrettyPrinter;
+import com.github.javaparser.printer.DefaultPrettyPrinter;
 
 import java.io.IOException;
+import java.nio.charset.Charset;
 import java.nio.file.FileVisitResult;
 import java.nio.file.Files;
 import java.nio.file.Path;
@@ -40,24 +61,31 @@ import static java.nio.file.FileVisitResult.*;
  * </ul>
  */
 public class SourceRoot {
+
     @FunctionalInterface
     public interface Callback {
+
         enum Result {
+
             SAVE, DONT_SAVE, TERMINATE
         }
 
         /**
          * @param localPath the path to the file that was parsed, relative to the source root path.
          * @param absolutePath the absolute path to the file that was parsed.
-         * @param result the result of of parsing the file.
+         * @param result the result of parsing the file.
          */
         Result process(Path localPath, Path absolutePath, ParseResult<CompilationUnit> result);
     }
 
     private final Path root;
+
     private final Map<Path, ParseResult<CompilationUnit>> cache = new ConcurrentHashMap<>();
+
     private ParserConfiguration parserConfiguration = new ParserConfiguration();
-    private Function<CompilationUnit, String> printer = new PrettyPrinter()::print;
+
+    private Function<CompilationUnit, String> printer = new DefaultPrettyPrinter()::print;
+
     private static final Pattern JAVA_IDENTIFIER = Pattern.compile("\\p{javaJavaIdentifierStart}\\p{javaJavaIdentifierPart}*");
 
     /**
@@ -100,9 +128,8 @@ public class SourceRoot {
         }
         final Path path = root.resolve(relativePath);
         Log.trace("Parsing %s", () -> path);
-        final ParseResult<CompilationUnit> result = new JavaParser(configuration)
-                .parse(COMPILATION_UNIT, provider(path));
-        result.getResult().ifPresent(cu -> cu.setStorage(path));
+        final ParseResult<CompilationUnit> result = new JavaParser(configuration).parse(COMPILATION_UNIT, provider(path, configuration.getCharacterEncoding()));
+        result.getResult().ifPresent(cu -> cu.setStorage(path, configuration.getCharacterEncoding()));
         cache.put(relativePath, result);
         return result;
     }
@@ -132,6 +159,7 @@ public class SourceRoot {
         logPackage(startPackage);
         final Path path = packageAbsolutePath(root, startPackage);
         Files.walkFileTree(path, new SimpleFileVisitor<Path>() {
+
             @Override
             public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                 if (!attrs.isDirectory() && file.toString().endsWith(".java")) {
@@ -149,12 +177,13 @@ public class SourceRoot {
         return getCache();
     }
 
-    private static boolean isSensibleDirectoryToEnter(Path dir) throws IOException {
+    boolean isSensibleDirectoryToEnter(Path dir) throws IOException {
         final String dirToEnter = dir.getFileName().toString();
         // Don't enter directories that cannot be packages.
         final boolean directoryIsAValidJavaIdentifier = JAVA_IDENTIFIER.matcher(dirToEnter).matches();
         // Don't enter directories that are hidden, assuming that people don't store source files in hidden directories.
-        if (Files.isHidden(dir) || !directoryIsAValidJavaIdentifier) {
+        // But we can enter in root directory even if the root directory is not considered as a valid java identifier
+        if (!root.equals(dir) && (Files.isHidden(dir) || !directoryIsAValidJavaIdentifier)) {
             Log.trace("Not processing directory \"%s\"", () -> dirToEnter);
             return false;
         }
@@ -191,10 +220,7 @@ public class SourceRoot {
             if (!attrs.isDirectory() && file.toString().endsWith(".java")) {
                 Path relative = root.relativize(file.getParent());
                 try {
-                    tryToParse(
-                            relative.toString(),
-                            file.getFileName().toString(),
-                            parserConfiguration);
+                    tryToParse(relative.toString(), file.getFileName().toString(), parserConfiguration);
                 } catch (IOException e) {
                     Log.error(e);
                 }
@@ -245,9 +271,9 @@ public class SourceRoot {
     private FileVisitResult callback(Path absolutePath, ParserConfiguration configuration, Callback callback) throws IOException {
         Path localPath = root.relativize(absolutePath);
         Log.trace("Parsing %s", () -> localPath);
-        ParseResult<CompilationUnit> result = new JavaParser(configuration).parse(COMPILATION_UNIT, provider(absolutePath));
-        result.getResult().ifPresent(cu -> cu.setStorage(absolutePath));
-        switch (callback.process(localPath, absolutePath, result)) {
+        ParseResult<CompilationUnit> result = new JavaParser(configuration).parse(COMPILATION_UNIT, provider(absolutePath, configuration.getCharacterEncoding()));
+        result.getResult().ifPresent(cu -> cu.setStorage(absolutePath, configuration.getCharacterEncoding()));
+        switch(callback.process(localPath, absolutePath, result)) {
             case SAVE:
                 result.getResult().ifPresent(cu -> save(cu, absolutePath));
             case DONT_SAVE:
@@ -266,8 +292,7 @@ public class SourceRoot {
      * @param startPackage The package containing the file
      * @param filename The name of the file
      */
-    public SourceRoot parse(String startPackage, String filename, ParserConfiguration configuration, Callback
-            callback) throws IOException {
+    public SourceRoot parse(String startPackage, String filename, ParserConfiguration configuration, Callback callback) throws IOException {
         assertNotNull(startPackage);
         assertNotNull(filename);
         assertNotNull(configuration);
@@ -299,6 +324,7 @@ public class SourceRoot {
         final Path path = packageAbsolutePath(root, startPackage);
         if (Files.exists(path)) {
             Files.walkFileTree(path, new SimpleFileVisitor<Path>() {
+
                 @Override
                 public FileVisitResult visitFile(Path absolutePath, BasicFileAttributes attrs) throws IOException {
                     if (!attrs.isDirectory() && absolutePath.toString().endsWith(".java")) {
@@ -371,7 +397,7 @@ public class SourceRoot {
      * @param startPackage files in this package and deeper are parsed. Pass "" to parse all files.
      */
     public SourceRoot parseParallelized(String startPackage, Callback callback) throws IOException {
-        return parseParallelized(startPackage, new ParserConfiguration(), callback);
+        return parseParallelized(startPackage, this.parserConfiguration, callback);
     }
 
     /**
@@ -382,7 +408,7 @@ public class SourceRoot {
      * this is much more memory efficient, but saveAll() won't work.
      */
     public SourceRoot parseParallelized(Callback callback) throws IOException {
-        return parseParallelized("", new ParserConfiguration(), callback);
+        return parseParallelized("", this.parserConfiguration, callback);
     }
 
     /**
@@ -396,10 +422,7 @@ public class SourceRoot {
         assertNotNull(compilationUnit);
         Log.trace("Adding new file %s.%s", () -> startPackage, () -> filename);
         final Path path = fileInPackageRelativePath(startPackage, filename);
-        final ParseResult<CompilationUnit> parseResult = new ParseResult<>(
-                compilationUnit,
-                new ArrayList<>(),
-                null);
+        final ParseResult<CompilationUnit> parseResult = new ParseResult<>(compilationUnit, new ArrayList<>(), null);
         cache.put(path, parseResult);
         return this;
     }
@@ -413,10 +436,7 @@ public class SourceRoot {
         if (compilationUnit.getStorage().isPresent()) {
             final Path path = compilationUnit.getStorage().get().getPath();
             Log.trace("Adding new file %s", () -> path);
-            final ParseResult<CompilationUnit> parseResult = new ParseResult<>(
-                    compilationUnit,
-                    new ArrayList<>(),
-                    null);
+            final ParseResult<CompilationUnit> parseResult = new ParseResult<>(compilationUnit, new ArrayList<>(), null);
             cache.put(path, parseResult);
         } else {
             throw new AssertionError("Files added with this method should have their path set.");
@@ -426,31 +446,53 @@ public class SourceRoot {
 
     /**
      * Save the given compilation unit to the given path.
+     * @param cu the compilation unit
+     * @param path the path of the java file
      */
     private SourceRoot save(CompilationUnit cu, Path path) {
+        return save(cu, path, parserConfiguration.getCharacterEncoding());
+    }
+
+    /**
+     * Save the given compilation unit to the given path.
+     * @param cu the compilation unit
+     * @param path the path of the java file
+     * @param encoding  the encoding to use while saving the file
+     */
+    private SourceRoot save(CompilationUnit cu, Path path, Charset encoding) {
         assertNotNull(cu);
         assertNotNull(path);
-        cu.setStorage(path);
+        cu.setStorage(path, encoding);
         cu.getStorage().get().save(printer);
         return this;
     }
 
     /**
      * Save all previously parsed files back to a new path.
+     * @param root the root of the java packages
+     * @param encoding the encoding to use while saving the file
      */
-    public SourceRoot saveAll(Path root) {
+    public SourceRoot saveAll(Path root, Charset encoding) {
         assertNotNull(root);
         Log.info("Saving all files (%s) to %s", cache::size, () -> root);
         for (Map.Entry<Path, ParseResult<CompilationUnit>> cu : cache.entrySet()) {
             final Path path = root.resolve(cu.getKey());
             if (cu.getValue().getResult().isPresent()) {
                 Log.trace("Saving %s", () -> path);
-                save(cu.getValue().getResult().get(), path);
+                save(cu.getValue().getResult().get(), path, encoding);
             }
         }
         return this;
     }
 
+    /**
+     * Save all previously parsed files back to a new path.
+     * @param root the root of the java packages
+     */
+    public SourceRoot saveAll(Path root) {
+        return saveAll(root, parserConfiguration.getCharacterEncoding());
+    }
+
     /**
      * Save all previously parsed files back to where they were found.
      */
@@ -458,6 +500,14 @@ public class SourceRoot {
         return saveAll(root);
     }
 
+    /**
+     * Save all previously parsed files back to where they were found, with the given encoding.
+     * @param encoding the encoding to use.
+     */
+    public SourceRoot saveAll(Charset encoding) {
+        return saveAll(root, encoding);
+    }
+
     /**
      * The Java files that have been parsed by this source root object, or have been added manually.
      */
@@ -470,10 +520,7 @@ public class SourceRoot {
      * added manually.
      */
     public List<CompilationUnit> getCompilationUnits() {
-        return cache.values().stream()
-                .filter(ParseResult::isSuccessful)
-                .map(p -> p.getResult().get())
-                .collect(Collectors.toList());
+        return cache.values().stream().filter(ParseResult::isSuccessful).map(p -> p.getResult().get()).collect(Collectors.toList());
     }
 
     /**
@@ -521,32 +568,35 @@ public class SourceRoot {
     private static class ParallelParse extends RecursiveAction {
 
         private static final long serialVersionUID = 1L;
-        private final Path path;
+
+        private final SourceRoot root;
+
         private final VisitFileCallback callback;
 
         ParallelParse(Path path, VisitFileCallback callback) {
-            this.path = path;
+            this.root = new SourceRoot(path);
             this.callback = callback;
         }
 
         @Override
         protected void compute() {
             final List<ParallelParse> walks = new ArrayList<>();
+            Path path = root.getRoot();
             try {
                 Files.walkFileTree(path, new SimpleFileVisitor<Path>() {
+
                     @Override
                     public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
-                        if (!SourceRoot.isSensibleDirectoryToEnter(dir)) {
+                        if (!root.isSensibleDirectoryToEnter(dir)) {
                             return SKIP_SUBTREE;
                         }
-                        if (!dir.equals(ParallelParse.this.path)) {
+                        if (!dir.equals(path)) {
                             ParallelParse w = new ParallelParse(dir, callback);
                             w.fork();
                             walks.add(w);
                             return SKIP_SUBTREE;
-                        } else {
-                            return CONTINUE;
                         }
+                        return CONTINUE;
                     }
 
                     @Override
@@ -557,13 +607,13 @@ public class SourceRoot {
             } catch (IOException e) {
                 Log.error(e);
             }
-
             for (ParallelParse w : walks) {
                 w.join();
             }
         }
 
         interface VisitFileCallback {
+
             FileVisitResult process(Path file, BasicFileAttributes attrs);
         }
     }
