@@ -1,21 +1,50 @@
+/*
+ * Copyright (C) 2015-2016 Federico Tomassetti
+ * Copyright (C) 2017-2024 The JavaParser Team.
+ *
+ * This file is part of JavaParser.
+ *
+ * JavaParser can be used either under the terms of
+ * a) the GNU Lesser General Public License as published by
+ *     the Free Software Foundation, either version 3 of the License, or
+ *     (at your option) any later version.
+ * b) the terms of the Apache License
+ *
+ * You should have received a copy of both licenses in LICENCE.LGPL and
+ * LICENCE.APACHE. Please refer to those files for details.
+ *
+ * JavaParser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ */
+
 package com.github.javaparser.symbolsolver;
 
+import static com.github.javaparser.resolution.Navigator.demandParentNode;
+
+import java.util.Optional;
+
 import com.github.javaparser.ast.CompilationUnit;
 import com.github.javaparser.ast.Node;
 import com.github.javaparser.ast.body.*;
 import com.github.javaparser.ast.expr.*;
+import com.github.javaparser.ast.stmt.CatchClause;
 import com.github.javaparser.ast.stmt.ExplicitConstructorInvocationStmt;
 import com.github.javaparser.ast.type.Type;
+import com.github.javaparser.ast.type.TypeParameter;
+import com.github.javaparser.quality.NotNull;
 import com.github.javaparser.resolution.SymbolResolver;
+import com.github.javaparser.resolution.TypeSolver;
 import com.github.javaparser.resolution.UnsolvedSymbolException;
 import com.github.javaparser.resolution.declarations.*;
+import com.github.javaparser.resolution.model.SymbolReference;
+import com.github.javaparser.resolution.model.Value;
 import com.github.javaparser.resolution.types.ResolvedPrimitiveType;
 import com.github.javaparser.resolution.types.ResolvedType;
 import com.github.javaparser.symbolsolver.javaparsermodel.JavaParserFacade;
 import com.github.javaparser.symbolsolver.javaparsermodel.JavaParserFactory;
 import com.github.javaparser.symbolsolver.javaparsermodel.declarations.*;
-import com.github.javaparser.symbolsolver.model.resolution.SymbolReference;
-import com.github.javaparser.symbolsolver.model.resolution.TypeSolver;
 
 /**
  * This implementation of the SymbolResolver wraps the functionality of the library to make them easily usable
@@ -30,6 +59,9 @@ import com.github.javaparser.symbolsolver.model.resolution.TypeSolver;
  */
 public class JavaSymbolSolver implements SymbolResolver {
 
+	/*
+	 * add possibility to resolve array.length #1695
+	 */
     private static class ArrayLengthValueDeclaration implements ResolvedValueDeclaration {
 
         private static final ArrayLengthValueDeclaration INSTANCE = new ArrayLengthValueDeclaration();
@@ -51,7 +83,7 @@ public class JavaSymbolSolver implements SymbolResolver {
 
     private TypeSolver typeSolver;
 
-    public JavaSymbolSolver(TypeSolver typeSolver) {
+    public JavaSymbolSolver(@NotNull TypeSolver typeSolver) {
         this.typeSolver = typeSolver;
     }
 
@@ -69,13 +101,13 @@ public class JavaSymbolSolver implements SymbolResolver {
             return resultClass.cast(new JavaParserMethodDeclaration((MethodDeclaration) node, typeSolver));
         }
         if (node instanceof ClassOrInterfaceDeclaration) {
-            ResolvedReferenceTypeDeclaration resolved = JavaParserFactory.toTypeDeclaration(node, typeSolver);
+            ResolvedReferenceTypeDeclaration resolved = toTypeDeclaration(node);
             if (resultClass.isInstance(resolved)) {
                 return resultClass.cast(resolved);
             }
         }
         if (node instanceof EnumDeclaration) {
-            ResolvedReferenceTypeDeclaration resolved = JavaParserFactory.toTypeDeclaration(node, typeSolver);
+            ResolvedReferenceTypeDeclaration resolved = toTypeDeclaration(node);
             if (resultClass.isInstance(resolved)) {
                 return resultClass.cast(resolved);
             }
@@ -93,7 +125,7 @@ public class JavaSymbolSolver implements SymbolResolver {
             ResolvedReferenceTypeDeclaration resolvedTypeDeclaration = resolveDeclaration(typeDeclaration, ResolvedReferenceTypeDeclaration.class);
             ResolvedConstructorDeclaration resolved = resolvedTypeDeclaration.getConstructors().stream()
                     .filter(c -> c instanceof JavaParserConstructorDeclaration)
-                    .filter(c -> ((JavaParserConstructorDeclaration) c).getWrappedNode() == constructorDeclaration)
+                    .filter(c -> ((JavaParserConstructorDeclaration<?>) c).getWrappedNode() == constructorDeclaration)
                     .findFirst()
                     .orElseThrow(() -> new RuntimeException("This constructor cannot be found in its parent. This seems wrong"));
             if (resultClass.isInstance(resolved)) {
@@ -101,7 +133,7 @@ public class JavaSymbolSolver implements SymbolResolver {
             }
         }
         if (node instanceof AnnotationDeclaration) {
-            ResolvedReferenceTypeDeclaration resolved = JavaParserFactory.toTypeDeclaration(node, typeSolver);
+            ResolvedReferenceTypeDeclaration resolved = toTypeDeclaration(node);
             if (resultClass.isInstance(resolved)) {
                 return resultClass.cast(resolved);
             }
@@ -166,6 +198,16 @@ public class JavaSymbolSolver implements SymbolResolver {
                 throw new UnsolvedSymbolException("We are unable to find the value declaration corresponding to " + node);
             }
         }
+        if (node instanceof MethodReferenceExpr) {
+            SymbolReference<ResolvedMethodDeclaration> result = JavaParserFacade.get(typeSolver).solve((MethodReferenceExpr) node);
+            if (result.isSolved()) {
+                if (resultClass.isInstance(result.getCorrespondingDeclaration())) {
+                    return resultClass.cast(result.getCorrespondingDeclaration());
+                }
+            } else {
+                throw new UnsolvedSymbolException("We are unable to find the method declaration corresponding to " + node);
+            }
+        }
         if (node instanceof FieldAccessExpr) {
             SymbolReference<? extends ResolvedValueDeclaration> result = JavaParserFacade.get(typeSolver).solve((FieldAccessExpr) node);
             if (result.isSolved()) {
@@ -173,7 +215,7 @@ public class JavaSymbolSolver implements SymbolResolver {
                     return resultClass.cast(result.getCorrespondingDeclaration());
                 }
             } else {
-                if (((FieldAccessExpr) node).getName().getId().equals("length")) {
+                if ("length".equals(((FieldAccessExpr) node).getName().getId())) {
                     ResolvedType scopeType = ((FieldAccessExpr) node).getScope().calculateResolvedType();
                     if (scopeType.isArray()) {
                         if (resultClass.isInstance(ArrayLengthValueDeclaration.INSTANCE)) {
@@ -204,23 +246,51 @@ public class JavaSymbolSolver implements SymbolResolver {
                 throw new UnsolvedSymbolException("We are unable to find the constructor declaration corresponding to " + node);
             }
         }
-        if (node instanceof Parameter) {
-            if (ResolvedParameterDeclaration.class.equals(resultClass)) {
-                Parameter parameter = (Parameter) node;
-                CallableDeclaration callableDeclaration = node.findAncestor(CallableDeclaration.class).get();
-                ResolvedMethodLikeDeclaration resolvedMethodLikeDeclaration;
-                if (callableDeclaration.isConstructorDeclaration()) {
-                    resolvedMethodLikeDeclaration = callableDeclaration.asConstructorDeclaration().resolve();
-                } else {
-                    resolvedMethodLikeDeclaration = callableDeclaration.asMethodDeclaration().resolve();
-                }
-                for (int i = 0; i < resolvedMethodLikeDeclaration.getNumberOfParams(); i++) {
-                    if (resolvedMethodLikeDeclaration.getParam(i).getName().equals(parameter.getNameAsString())) {
-                        return resultClass.cast(resolvedMethodLikeDeclaration.getParam(i));
-                    }
-                }
-            }
-        }
+		if (node instanceof Parameter) {
+			if (ResolvedParameterDeclaration.class.equals(resultClass)) {
+				Parameter parameter = (Parameter) node;
+				Optional<Node> parentNode = node.getParentNode();
+				if (!parentNode.isPresent()) {
+					throw new UnsolvedSymbolException(
+							"We are unable to resolve the parameter declaration corresponding to " + node);
+				}
+				Node parent = (Node) parentNode.get();
+				if (parent instanceof ConstructorDeclaration) {
+					Optional<ResolvedParameterDeclaration> resolvedParameterDeclaration = resolveParameterDeclaration(
+							((ConstructorDeclaration) parent).resolve(), parameter);
+					return resolvedParameterDeclaration.map(rpd -> resultClass.cast(rpd))
+							.orElseThrow(() -> new UnsolvedSymbolException(
+									"We are unable to resolve the parameter declaration corresponding to " + node));
+				} else if (parent instanceof MethodDeclaration) {
+					Optional<ResolvedParameterDeclaration> resolvedParameterDeclaration = resolveParameterDeclaration(
+							((MethodDeclaration) parent).resolve(), parameter);
+					return resolvedParameterDeclaration.map(rpd -> resultClass.cast(rpd))
+							.orElseThrow(() -> new UnsolvedSymbolException(
+									"We are unable to resolve the parameter declaration corresponding to " + node));
+				} else if (parent instanceof RecordDeclaration) {
+					Optional<ResolvedParameterDeclaration> resolvedParameterDeclaration = resolveParameterDeclaration(
+							((RecordDeclaration) parent).resolve(), parameter);
+					return resolvedParameterDeclaration.map(rpd -> resultClass.cast(rpd))
+							.orElseThrow(() -> new UnsolvedSymbolException(
+									"We are unable to resolve the parameter declaration corresponding to " + node));
+				} else if (parent instanceof LambdaExpr) {
+					Optional<ResolvedParameterDeclaration> resolvedParameterDeclaration = resolveParameterDeclaration(
+							parameter);
+					return resolvedParameterDeclaration.map(rpd -> resultClass.cast(rpd))
+							.orElseThrow(() -> new UnsolvedSymbolException(
+									"We are unable to resolve the parameter declaration corresponding to " + node));
+				} else if (parent instanceof CatchClause) {
+					Optional<ResolvedParameterDeclaration> resolvedParameterDeclaration = resolveParameterDeclaration(
+							parameter);
+					return resolvedParameterDeclaration.map(rpd -> resultClass.cast(rpd))
+							.orElseThrow(() -> new UnsolvedSymbolException(
+									"We are unable to resolve the parameter declaration corresponding to " + node));
+				} else {
+					throw new UnsolvedSymbolException(
+							"We are unable to resolve the parameter declaration corresponding to " + node);
+				}
+			}
+		}
         if (node instanceof AnnotationExpr) {
             SymbolReference<ResolvedAnnotationDeclaration> result = JavaParserFacade.get(typeSolver).solve((AnnotationExpr) node);
             if (result.isSolved()) {
@@ -231,13 +301,96 @@ public class JavaSymbolSolver implements SymbolResolver {
                 throw new UnsolvedSymbolException("We are unable to find the annotation declaration corresponding to " + node);
             }
         }
+        if (node instanceof PatternExpr) {
+            SymbolReference<? extends ResolvedValueDeclaration> result = JavaParserFacade.get(typeSolver).solve((PatternExpr) node);
+            if (result.isSolved()) {
+                if (resultClass.isInstance(result.getCorrespondingDeclaration())) {
+                    return resultClass.cast(result.getCorrespondingDeclaration());
+                }
+            } else {
+                throw new UnsolvedSymbolException("We are unable to find the method declaration corresponding to " + node);
+            }
+        }
         throw new UnsupportedOperationException("Unable to find the declaration of type " + resultClass.getSimpleName()
                 + " from " + node.getClass().getSimpleName());
     }
+    
+    /*
+     * Resolves constructor or method parameter
+     */
+	private Optional<ResolvedParameterDeclaration> resolveParameterDeclaration(
+			ResolvedMethodLikeDeclaration resolvedMethodLikeDeclaration, Parameter parameter) {
+		for (int i = 0; i < resolvedMethodLikeDeclaration.getNumberOfParams(); i++) {
+			if (resolvedMethodLikeDeclaration.getParam(i).getName().equals(parameter.getNameAsString())) {
+				return Optional.of(resolvedMethodLikeDeclaration.getParam(i));
+			}
+		}
+		return Optional.empty();
+	}
+	
+	/*
+	 * Resolves record parameter
+	 */
+	private Optional<ResolvedParameterDeclaration> resolveParameterDeclaration(
+			ResolvedReferenceTypeDeclaration resolvedReferenceTypeDeclaration, Parameter parameter) {
+		ResolvedFieldDeclaration rfd = resolvedReferenceTypeDeclaration.getField(parameter.getNameAsString());
+		if (rfd == null) return Optional.empty();
+		ResolvedParameterDeclaration  resolvedParameterDeclaration = new ResolvedParameterDeclaration() {
 
+			@Override
+			public ResolvedType getType() {
+				return rfd.getType();
+			}
+
+			@Override
+			public String getName() {
+				return parameter.getNameAsString();
+			}
+
+			@Override
+			public boolean isVariadic() {
+				return parameter.isVarArgs();
+			}
+			
+		};
+		return Optional.of(resolvedParameterDeclaration);
+	}
+	
+	/*
+	 * Resolves lambda expression parameters and catch clause parameters
+	 */
+	private Optional<ResolvedParameterDeclaration> resolveParameterDeclaration(Parameter parameter) {
+		ResolvedParameterDeclaration resolvedParameterDeclaration = new ResolvedParameterDeclaration() {
+
+			@Override
+			public ResolvedType getType() {
+				Node parentNode = parameter.getParentNode().get();
+				if (parameter.getType().isUnknownType() && parentNode instanceof LambdaExpr) {
+					Optional<Value> value = JavaParserFactory.getContext(parentNode, typeSolver)
+							.solveSymbolAsValue(parameter.getNameAsString());
+					return value.map(v -> v.getType()).orElseThrow(() -> new UnsolvedSymbolException(
+							"We are unable to resolve the parameter declaration corresponding to " + parameter));
+				}
+				return JavaParserFacade.get(typeSolver).convertToUsage(parameter.getType());
+			}
+
+			@Override
+			public String getName() {
+				return parameter.getNameAsString();
+			}
+
+			@Override
+			public boolean isVariadic() {
+				return parameter.isVarArgs();
+			}
+
+		};
+		return Optional.of(resolvedParameterDeclaration);
+	}
+	
     @Override
     public <T> T toResolvedType(Type javaparserType, Class<T> resultClass) {
-        ResolvedType resolvedType = JavaParserFacade.get(typeSolver).convertToUsage(javaparserType, javaparserType);
+        ResolvedType resolvedType = JavaParserFacade.get(typeSolver).convertToUsage(javaparserType);
         if (resultClass.isInstance(resolvedType)) {
             return resultClass.cast(resolvedType);
         }
@@ -249,4 +402,27 @@ public class JavaSymbolSolver implements SymbolResolver {
     public ResolvedType calculateType(Expression expression) {
         return JavaParserFacade.get(typeSolver).getType(expression);
     }
+
+    @Override
+    public ResolvedReferenceTypeDeclaration toTypeDeclaration(Node node) {
+        if (node instanceof ClassOrInterfaceDeclaration) {
+            if (((ClassOrInterfaceDeclaration) node).isInterface()) {
+                return new JavaParserInterfaceDeclaration((ClassOrInterfaceDeclaration) node, typeSolver);
+            }
+            return new JavaParserClassDeclaration((ClassOrInterfaceDeclaration) node, typeSolver);
+        }
+        if (node instanceof TypeParameter) {
+            return new JavaParserTypeParameter((TypeParameter) node, typeSolver);
+        }
+        if (node instanceof EnumDeclaration) {
+            return new JavaParserEnumDeclaration((EnumDeclaration) node, typeSolver);
+        }
+        if (node instanceof AnnotationDeclaration) {
+            return new JavaParserAnnotationDeclaration((AnnotationDeclaration) node, typeSolver);
+        }
+        if (node instanceof EnumConstantDeclaration) {
+            return new JavaParserEnumDeclaration((EnumDeclaration) demandParentNode(node), typeSolver);
+        }
+        throw new IllegalArgumentException("Cannot get a reference type declaration from " + node.getClass().getCanonicalName());
+    }
 }
