@@ -1,63 +1,71 @@
 /*
- * Copyright 2016 Federico Tomassetti
+ * Copyright (C) 2015-2016 Federico Tomassetti
+ * Copyright (C) 2017-2024 The JavaParser Team.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * This file is part of JavaParser.
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * JavaParser can be used either under the terms of
+ * a) the GNU Lesser General Public License as published by
+ *     the Free Software Foundation, either version 3 of the License, or
+ *     (at your option) any later version.
+ * b) the terms of the Apache License
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * You should have received a copy of both licenses in LICENCE.LGPL and
+ * LICENCE.APACHE. Please refer to those files for details.
+ *
+ * JavaParser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
  */
 
 package com.github.javaparser.symbolsolver.javaparsermodel.declarations;
 
+import java.util.*;
+import java.util.stream.Collectors;
+
 import com.github.javaparser.ast.AccessSpecifier;
-import com.github.javaparser.ast.Modifier;
 import com.github.javaparser.ast.Node;
-import com.github.javaparser.ast.body.*;
+import com.github.javaparser.ast.body.BodyDeclaration;
+import com.github.javaparser.ast.body.ClassOrInterfaceDeclaration;
+import com.github.javaparser.ast.body.MethodDeclaration;
 import com.github.javaparser.ast.type.ClassOrInterfaceType;
+import com.github.javaparser.resolution.Context;
 import com.github.javaparser.resolution.MethodUsage;
+import com.github.javaparser.resolution.TypeSolver;
 import com.github.javaparser.resolution.UnsolvedSymbolException;
 import com.github.javaparser.resolution.declarations.*;
+import com.github.javaparser.resolution.model.SymbolReference;
+import com.github.javaparser.resolution.model.typesystem.LazyType;
+import com.github.javaparser.resolution.model.typesystem.ReferenceTypeImpl;
 import com.github.javaparser.resolution.types.ResolvedReferenceType;
 import com.github.javaparser.resolution.types.ResolvedType;
-import com.github.javaparser.symbolsolver.core.resolution.Context;
 import com.github.javaparser.symbolsolver.core.resolution.MethodUsageResolutionCapability;
+import com.github.javaparser.symbolsolver.core.resolution.SymbolResolutionCapability;
 import com.github.javaparser.symbolsolver.javaparsermodel.JavaParserFacade;
 import com.github.javaparser.symbolsolver.javaparsermodel.JavaParserFactory;
 import com.github.javaparser.symbolsolver.logic.AbstractClassDeclaration;
-import com.github.javaparser.symbolsolver.model.resolution.SymbolReference;
-import com.github.javaparser.symbolsolver.model.resolution.TypeSolver;
-import com.github.javaparser.symbolsolver.model.typesystem.LazyType;
-import com.github.javaparser.symbolsolver.model.typesystem.ReferenceTypeImpl;
 import com.github.javaparser.symbolsolver.resolution.SymbolSolver;
 
-import java.util.*;
-import java.util.stream.Collectors;
-
 /**
  * @author Federico Tomassetti
  */
-public class JavaParserClassDeclaration extends AbstractClassDeclaration implements MethodUsageResolutionCapability {
+public class JavaParserClassDeclaration extends AbstractClassDeclaration
+        implements MethodUsageResolutionCapability, SymbolResolutionCapability {
 
     ///
     /// Fields
     ///
 
     private TypeSolver typeSolver;
-    private com.github.javaparser.ast.body.ClassOrInterfaceDeclaration wrappedNode;
+    private ClassOrInterfaceDeclaration wrappedNode;
     private JavaParserTypeAdapter<ClassOrInterfaceDeclaration> javaParserTypeAdapter;
 
     ///
     /// Constructors
     ///
 
-    public JavaParserClassDeclaration(com.github.javaparser.ast.body.ClassOrInterfaceDeclaration wrappedNode,
+    public JavaParserClassDeclaration(ClassOrInterfaceDeclaration wrappedNode,
                                       TypeSolver typeSolver) {
         if (wrappedNode.isInterface()) {
             throw new IllegalArgumentException("Interface given");
@@ -78,9 +86,7 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
 
         JavaParserClassDeclaration that = (JavaParserClassDeclaration) o;
 
-        if (!wrappedNode.equals(that.wrappedNode)) return false;
-
-        return true;
+        return wrappedNode.equals(that.wrappedNode);
     }
 
     @Override
@@ -91,8 +97,8 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
     @Override
     public String toString() {
         return "JavaParserClassDeclaration{" +
-               "wrappedNode=" + wrappedNode +
-               '}';
+                "wrappedNode=" + wrappedNode +
+                '}';
     }
 
     ///
@@ -103,35 +109,47 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
     public List<ResolvedFieldDeclaration> getAllFields() {
         List<ResolvedFieldDeclaration> fields = javaParserTypeAdapter.getFieldsForDeclaredVariables();
 
-        getAncestors(true).forEach(ancestor -> ancestor.getTypeDeclaration().getAllFields().forEach(f -> {
-            fields.add(new ResolvedFieldDeclaration() {
+        getAncestors(true).stream().filter(ancestor -> ancestor.getTypeDeclaration().isPresent())
+                .forEach(ancestor -> ancestor.getTypeDeclaration().get().getAllFields()
+                        .forEach(f -> {
+                            fields.add(new ResolvedFieldDeclaration() {
 
-                @Override
-                public AccessSpecifier accessSpecifier() {
-                    return f.accessSpecifier();
-                }
+                                @Override
+                                public AccessSpecifier accessSpecifier() {
+                                    return f.accessSpecifier();
+                                }
 
-                @Override
-                public String getName() {
-                    return f.getName();
-                }
+                                @Override
+                                public String getName() {
+                                    return f.getName();
+                                }
 
-                @Override
-                public ResolvedType getType() {
-                    return ancestor.useThisTypeParametersOnTheGivenType(f.getType());
-                }
+                                @Override
+                                public ResolvedType getType() {
+                                    return ancestor.useThisTypeParametersOnTheGivenType(f.getType());
+                                }
 
-                @Override
-                public boolean isStatic() {
-                    return f.isStatic();
-                }
+                                @Override
+                                public boolean isStatic() {
+                                    return f.isStatic();
+                                }
 
-                @Override
-                public ResolvedTypeDeclaration declaringType() {
-                    return f.declaringType();
-                }
-            });
-        }));
+                                @Override
+                                public boolean isVolatile() {
+                                    return f.isVolatile();
+                                }
+
+                                @Override
+                                public ResolvedTypeDeclaration declaringType() {
+                                    return f.declaringType();
+                                }
+
+                                @Override
+                                public Optional<Node> toAst() {
+                                    return f.toAst();
+                                }
+                            });
+                        }));
 
         return fields;
     }
@@ -170,17 +188,22 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
     }
 
     @Override
-    public ResolvedReferenceType getSuperClass() {
-        if (wrappedNode.getExtendedTypes().isEmpty()) {
-            return object();
-        } else {
-            return toReferenceType(wrappedNode.getExtendedTypes().get(0));
+    public Optional<ResolvedReferenceType> getSuperClass() {
+        if (isJavaLangObject()) {
+            // If this is java.lang.Object, it has no super class.
+            return Optional.empty();
         }
+        if (wrappedNode.getExtendedTypes().isEmpty()) {
+            // All objects implicitly extend java.lang.Object -- inject it here (only when this isn't java.lang.Object)
+            return Optional.of(object());
+        }
+        return Optional.of(toReferenceType(wrappedNode.getExtendedTypes().getFirst().get()));
     }
 
     @Override
     public List<ResolvedReferenceType> getInterfaces() {
         List<ResolvedReferenceType> interfaces = new ArrayList<>();
+        // TODO FIXME: Remove null check -- should be an empty list...
         if (wrappedNode.getImplementedTypes() != null) {
             for (ClassOrInterfaceType t : wrappedNode.getImplementedTypes()) {
                 interfaces.add(toReferenceType(t));
@@ -199,6 +222,14 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
         return AstResolutionUtils.hasDirectlyAnnotation(wrappedNode, typeSolver, canonicalName);
     }
 
+    /*
+     * Returns a set of the declared annotation on this type
+     */
+    @Override
+    public Set<ResolvedAnnotationDeclaration> getDeclaredAnnotations() {
+        return javaParserTypeAdapter.getDeclaredAnnotations();
+    }
+
     @Override
     public boolean isInterface() {
         return wrappedNode.isInterface();
@@ -235,17 +266,21 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
         if (this.getQualifiedName().equals(other.getQualifiedName())) {
             return true;
         }
-        ResolvedClassDeclaration superclass = (ResolvedClassDeclaration) getSuperClass().getTypeDeclaration();
-        if (superclass != null) {
-            // We want to avoid infinite recursion in case of Object having Object as ancestor
-            if (Object.class.getCanonicalName().equals(superclass.getQualifiedName())) {
-                return true;
-            }
-            if (superclass.canBeAssignedTo(other)) {
-                return true;
+
+        Optional<ResolvedReferenceType> optionalSuperClass = getSuperClass();
+        if (optionalSuperClass.isPresent()) {
+            Optional<ResolvedReferenceTypeDeclaration> optionalSuperclassTypeDeclaration = optionalSuperClass.get().getTypeDeclaration();
+            if (optionalSuperclassTypeDeclaration.isPresent()) {
+                ResolvedReferenceTypeDeclaration superclassTypeDeclaration = optionalSuperclassTypeDeclaration.get();
+                if (superclassTypeDeclaration != this && superclassTypeDeclaration.isClass()) {
+                    if (superclassTypeDeclaration.asClass().canBeAssignedTo(other)) {
+                        return true;
+                    }
+                }
             }
         }
 
+        // TODO FIXME: Remove null check -- should be an empty list...
         if (this.wrappedNode.getImplementedTypes() != null) {
             for (ClassOrInterfaceType type : wrappedNode.getImplementedTypes()) {
                 ResolvedReferenceTypeDeclaration ancestor = (ResolvedReferenceTypeDeclaration) new SymbolSolver(typeSolver).solveType(type);
@@ -278,12 +313,14 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
             return ref;
         }
 
-        String prefix = wrappedNode.getName() + ".";
+        String prefix = wrappedNode.getName().asString() + ".";
         if (name.startsWith(prefix) && name.length() > prefix.length()) {
             return new JavaParserClassDeclaration(this.wrappedNode, typeSolver).solveType(name.substring(prefix.length()));
         }
 
-        return getContext().getParent().solveType(name);
+        return getContext().getParent()
+                .orElseThrow(() -> new RuntimeException("Parent context unexpectedly empty."))
+                .solveType(name);
     }
 
     @Override
@@ -292,45 +329,58 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
         return getContext().solveMethod(name, argumentsTypes, staticOnly);
     }
 
+    @Override
+    public SymbolReference<? extends ResolvedValueDeclaration> solveSymbol(String name, TypeSolver typeSolver) {
+        return getContext().solveSymbol(name);
+    }
+
     @Override
     public List<ResolvedReferenceType> getAncestors(boolean acceptIncompleteList) {
         List<ResolvedReferenceType> ancestors = new ArrayList<>();
 
         // We want to avoid infinite recursion in case of Object having Object as ancestor
-        if (!(Object.class.getCanonicalName().equals(getQualifiedName()))) {
-            ResolvedReferenceType superclass;
-            try {
-                superclass = getSuperClass();
-            } catch (UnsolvedSymbolException e) {
-                if (acceptIncompleteList) {
-                    // in case we could not resolve the super class, we may still be able to resolve (some of) the
-                    // implemented interfaces and so we continue gracefully with an (incomplete) list of ancestors
-                    superclass = null;
-                } else {
-                    throw e;
+        if (this.isJavaLangObject()) {
+            return ancestors;
+        }
+
+        Optional<String> qualifiedName = wrappedNode.getFullyQualifiedName();
+        if (!qualifiedName.isPresent()) {
+            return ancestors;
+        }
+
+        try {
+            // If a superclass is found, add it as an ancestor
+            Optional<ResolvedReferenceType> superClass = getSuperClass();
+            if (superClass.isPresent()) {
+                if (isAncestor(superClass.get(), qualifiedName.get())) {
+                    ancestors.add(superClass.get());
                 }
             }
-            if (superclass != null) {
-                ancestors.add(superclass);
+        } catch (UnsolvedSymbolException e) {
+            // in case we could not resolve the super class, we may still be able to resolve (some of) the
+            // implemented interfaces and so we continue gracefully with an (incomplete) list of ancestors
+
+            if (!acceptIncompleteList) {
+                // Only throw if an incomplete ancestor list is unacceptable.
+                throw e;
             }
-            if (wrappedNode.getImplementedTypes() != null) {
-                for (ClassOrInterfaceType implemented : wrappedNode.getImplementedTypes()) {
-                    ResolvedReferenceType ancestor;
-                    try {
-                        ancestor = toReferenceType(implemented);
-                    } catch (UnsolvedSymbolException e) {
-                        if (acceptIncompleteList) {
-                            // in case we could not resolve some implemented interface, we may still be able to resolve the
-                            // extended class or (some of) the other implemented interfaces and so we continue gracefully
-                            // with an (incomplete) list of ancestors
-                            ancestor = null;
-                        } else {
-                            throw e;
-                        }
-                    }
-                    if (ancestor != null) {
-                        ancestors.add(ancestor);
-                    }
+        }
+
+        for (ClassOrInterfaceType implemented : wrappedNode.getImplementedTypes()) {
+            try {
+                // If an implemented interface is found, add it as an ancestor
+                ResolvedReferenceType rrt = toReferenceType(implemented);
+                if (isAncestor(rrt, qualifiedName.get())) {
+                    ancestors.add(rrt);
+                }
+            } catch (UnsolvedSymbolException e) {
+                // in case we could not resolve some implemented interface, we may still be able to resolve the
+                // extended class or (some of) the other implemented interfaces and so we continue gracefully
+                // with an (incomplete) list of ancestors
+
+                if (!acceptIncompleteList) {
+                    // Only throw if an incomplete ancestor list is unacceptable.
+                    throw e;
                 }
             }
         }
@@ -338,12 +388,22 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
         return ancestors;
     }
 
+    private boolean isAncestor(ResolvedReferenceType candidateAncestor, String ownQualifiedName) {
+        Optional<ResolvedReferenceTypeDeclaration> resolvedReferenceTypeDeclaration = candidateAncestor.getTypeDeclaration();
+        if (resolvedReferenceTypeDeclaration.isPresent()) {
+            ResolvedTypeDeclaration rtd = resolvedReferenceTypeDeclaration.get().asType();
+            // do not consider an inner or nested class as an ancestor
+            return !rtd.hasInternalType(ownQualifiedName);
+        }
+        return false;
+    }
+
     @Override
     public Set<ResolvedMethodDeclaration> getDeclaredMethods() {
         Set<ResolvedMethodDeclaration> methods = new HashSet<>();
         for (BodyDeclaration<?> member : wrappedNode.getMembers()) {
-            if (member instanceof com.github.javaparser.ast.body.MethodDeclaration) {
-                methods.add(new JavaParserMethodDeclaration((com.github.javaparser.ast.body.MethodDeclaration) member, typeSolver));
+            if (member instanceof MethodDeclaration) {
+                methods.add(new JavaParserMethodDeclaration((MethodDeclaration) member, typeSolver));
             }
         }
         return methods;
@@ -361,7 +421,7 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
      *
      * @return A visitable JavaParser node wrapped by this object.
      */
-    public com.github.javaparser.ast.body.ClassOrInterfaceDeclaration getWrappedNode() {
+    public ClassOrInterfaceDeclaration getWrappedNode() {
         return wrappedNode;
     }
 
@@ -381,18 +441,13 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
 
     @Override
     protected ResolvedReferenceType object() {
-        return new ReferenceTypeImpl(typeSolver.solveType(Object.class.getCanonicalName()), typeSolver);
+        ResolvedReferenceTypeDeclaration solvedJavaLangObject = typeSolver.getSolvedJavaLangObject();
+        return new ReferenceTypeImpl(solvedJavaLangObject);
     }
 
     @Override
     public Set<ResolvedReferenceTypeDeclaration> internalTypes() {
-        Set<ResolvedReferenceTypeDeclaration> res = new HashSet<>();
-        for (BodyDeclaration<?> member : this.wrappedNode.getMembers()) {
-            if (member instanceof com.github.javaparser.ast.body.TypeDeclaration) {
-                res.add(JavaParserFacade.get(typeSolver).getTypeDeclaration((com.github.javaparser.ast.body.TypeDeclaration) member));
-            }
-        }
-        return res;
+        return javaParserTypeAdapter.internalTypes();
     }
 
     @Override
@@ -411,6 +466,8 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
             className = classOrInterfaceType.getScope().get().toString() + "." + className;
         }
         SymbolReference<ResolvedTypeDeclaration> ref = solveType(className);
+
+        // If unable to solve by the class name alone, attempt to qualify it.
         if (!ref.isSolved()) {
             Optional<ClassOrInterfaceType> localScope = classOrInterfaceType.getScope();
             if (localScope.isPresent()) {
@@ -418,15 +475,21 @@ public class JavaParserClassDeclaration extends AbstractClassDeclaration impleme
                 ref = solveType(localName);
             }
         }
+
+        // If still unable to resolve, throw an exception.
         if (!ref.isSolved()) {
             throw new UnsolvedSymbolException(classOrInterfaceType.getName().getId());
         }
+
         if (!classOrInterfaceType.getTypeArguments().isPresent()) {
-            return new ReferenceTypeImpl(ref.getCorrespondingDeclaration().asReferenceType(), typeSolver);
+            return new ReferenceTypeImpl(ref.getCorrespondingDeclaration().asReferenceType());
         }
+
         List<ResolvedType> superClassTypeParameters = classOrInterfaceType.getTypeArguments().get()
-                                                              .stream().map(ta -> new LazyType(v -> JavaParserFacade.get(typeSolver).convert(ta, ta)))
-                                                              .collect(Collectors.toList());
-        return new ReferenceTypeImpl(ref.getCorrespondingDeclaration().asReferenceType(), superClassTypeParameters, typeSolver);
+                .stream()
+                .map(ta -> new LazyType(v -> JavaParserFacade.get(typeSolver).convert(ta, ta)))
+                .collect(Collectors.toList());
+
+        return new ReferenceTypeImpl(ref.getCorrespondingDeclaration().asReferenceType(), superClassTypeParameters);
     }
 }
