@@ -1,24 +1,42 @@
-package com.github.javaparser.symbolsolver.resolution.typeinference;
+/*
+ * Copyright (C) 2015-2016 Federico Tomassetti
+ * Copyright (C) 2017-2024 The JavaParser Team.
+ *
+ * This file is part of JavaParser.
+ *
+ * JavaParser can be used either under the terms of
+ * a) the GNU Lesser General Public License as published by
+ *     the Free Software Foundation, either version 3 of the License, or
+ *     (at your option) any later version.
+ * b) the terms of the Apache License
+ *
+ * You should have received a copy of both licenses in LICENCE.LGPL and
+ * LICENCE.APACHE. Please refer to those files for details.
+ *
+ * JavaParser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ */
 
-import com.github.javaparser.ast.expr.*;
-import com.github.javaparser.ast.type.UnknownType;
-import com.github.javaparser.resolution.MethodUsage;
-import com.github.javaparser.resolution.declarations.ResolvedInterfaceDeclaration;
-import com.github.javaparser.resolution.declarations.ResolvedMethodDeclaration;
-import com.github.javaparser.resolution.declarations.ResolvedTypeParameterDeclaration;
-import com.github.javaparser.resolution.types.ResolvedType;
-import com.github.javaparser.symbolsolver.javaparsermodel.JavaParserFacade;
-import com.github.javaparser.symbolsolver.model.resolution.TypeSolver;
-import com.github.javaparser.symbolsolver.model.typesystem.ReferenceTypeImpl;
-import com.github.javaparser.symbolsolver.resolution.typeinference.bounds.SubtypeOfBound;
-import com.github.javaparser.symbolsolver.resolution.typeinference.bounds.ThrowsBound;
-import com.github.javaparser.symbolsolver.resolution.typeinference.constraintformulas.ExpressionCompatibleWithType;
+package com.github.javaparser.symbolsolver.resolution.typeinference;
 
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Optional;
 
-import static com.github.javaparser.symbolsolver.resolution.typeinference.ExpressionHelper.isStandaloneExpression;
+import com.github.javaparser.ast.expr.*;
+import com.github.javaparser.resolution.MethodUsage;
+import com.github.javaparser.resolution.TypeSolver;
+import com.github.javaparser.resolution.declarations.ResolvedInterfaceDeclaration;
+import com.github.javaparser.resolution.declarations.ResolvedMethodDeclaration;
+import com.github.javaparser.resolution.declarations.ResolvedTypeParameterDeclaration;
+import com.github.javaparser.resolution.model.typesystem.ReferenceTypeImpl;
+import com.github.javaparser.resolution.types.ResolvedType;
+import com.github.javaparser.symbolsolver.javaparsermodel.JavaParserFacade;
+import com.github.javaparser.symbolsolver.resolution.typeinference.bounds.SubtypeOfBound;
+import com.github.javaparser.symbolsolver.resolution.typeinference.bounds.ThrowsBound;
+import com.github.javaparser.symbolsolver.resolution.typeinference.constraintformulas.ExpressionCompatibleWithType;
 
 /**
  * The API exposed by the TypeInference subsystem.
@@ -35,7 +53,7 @@ public class TypeInference {
             throw new NullPointerException();
         }
         this.typeSolver = typeSolver;
-        this.object = new ReferenceTypeImpl(typeSolver.solveType(Object.class.getCanonicalName()), typeSolver);
+        this.object = new ReferenceTypeImpl(typeSolver.getSolvedJavaLangObject());
     }
 
     ///
@@ -47,9 +65,8 @@ public class TypeInference {
         Optional<InstantiationSet> instantiationSetOpt = typeInference.instantiationInference(call, methodDeclaration);
         if (instantiationSetOpt.isPresent()) {
             return instantiationSetToMethodUsage(methodDeclaration, instantiationSetOpt.get());
-        } else {
-            throw new IllegalArgumentException();
         }
+        throw new IllegalArgumentException();
     }
 
     ///
@@ -548,8 +565,7 @@ public class TypeInference {
     }
 
     private boolean appearInThrowsClause(ResolvedTypeParameterDeclaration p, ResolvedMethodDeclaration methodDeclaration) {
-        for (int j=0;j<methodDeclaration.getNumberOfSpecifiedExceptions();j++) {
-            ResolvedType thrownType = methodDeclaration.getSpecifiedException(j);
+        for (ResolvedType thrownType : methodDeclaration.getSpecifiedExceptions()) {
             if (thrownType.isTypeVariable() && thrownType.asTypeVariable().asTypeParameter().equals(p)) {
                 return true;
             }
@@ -566,7 +582,7 @@ public class TypeInference {
     }
 
     private boolean isImplicitlyTyped(LambdaExpr lambdaExpr) {
-        return lambdaExpr.getParameters().stream().anyMatch(p -> p.getType() instanceof UnknownType);
+        return lambdaExpr.getParameters().stream().anyMatch(p -> p.getType().isUnknownType());
     }
 
     private boolean isInexact(MethodReferenceExpr methodReferenceExpr) {
@@ -579,7 +595,7 @@ public class TypeInference {
         //
         // - An implicitly typed lambda expression (§15.27.1).
 
-        if (argument instanceof LambdaExpr) {
+        if (argument.isLambdaExpr()) {
             LambdaExpr lambdaExpr = (LambdaExpr)argument;
             if (isImplicitlyTyped(lambdaExpr)) {
                 return false;
@@ -588,7 +604,7 @@ public class TypeInference {
 
         // - An inexact method reference expression (§15.13.1).
 
-        if (argument instanceof MethodReferenceExpr) {
+        if (argument.isMethodReferenceExpr()) {
             MethodReferenceExpr methodReferenceExpr = (MethodReferenceExpr)argument;
             if (isInexact(methodReferenceExpr)) {
                 return false;
@@ -599,37 +615,37 @@ public class TypeInference {
         //   explicitly typed lambda expression or an exact method reference expression for which the
         //   corresponding target type (as derived from the signature of m) is a type parameter of m.
 
-        if (argument instanceof LambdaExpr) {
+        if (argument.isLambdaExpr()) {
             throw new UnsupportedOperationException();
         }
 
-        if (argument instanceof MethodReferenceExpr) {
+        if (argument.isMethodReferenceExpr()) {
             throw new UnsupportedOperationException();
         }
 
         // - An explicitly typed lambda expression whose body is an expression that is not pertinent to applicability.
 
-        if (argument instanceof LambdaExpr) {
+        if (argument.isLambdaExpr()) {
             throw new UnsupportedOperationException();
         }
 
         // - An explicitly typed lambda expression whose body is a block, where at least one result expression is not
         //   pertinent to applicability.
 
-        if (argument instanceof LambdaExpr) {
+        if (argument.isLambdaExpr()) {
             throw new UnsupportedOperationException();
         }
 
         // - A parenthesized expression (§15.8.5) whose contained expression is not pertinent to applicability.
 
-        if (argument instanceof EnclosedExpr) {
+        if (argument.isEnclosedExpr()) {
             EnclosedExpr enclosedExpr = (EnclosedExpr)argument;
             return isPertinentToApplicability(enclosedExpr.getInner());
         }
 
         // - A conditional expression (§15.25) whose second or third operand is not pertinent to applicability.
 
-        if (argument instanceof ConditionalExpr) {
+        if (argument.isConditionalExpr()) {
             ConditionalExpr conditionalExpr = (ConditionalExpr)argument;
             return isPertinentToApplicability(conditionalExpr.getThenExpr()) &&
                     isPertinentToApplicability(conditionalExpr.getElseExpr());
@@ -654,11 +670,11 @@ public class TypeInference {
             Expression ei = es.get(i);
             ResolvedType fi = Fs.get(i);
             if (isPertinentToApplicability(ei)) {
-                if (isStandaloneExpression(ei) && JavaParserFacade.get(typeSolver).getType(ei).isPrimitive()
+                if (ei.isStandaloneExpression() && JavaParserFacade.get(typeSolver).getType(ei).isPrimitive()
                         && fi.isReferenceType()) {
                     return Optional.empty();
                 }
-                if (fi.isPrimitive() && (!isStandaloneExpression(ei) || !JavaParserFacade.get(typeSolver).getType(ei).isPrimitive())) {
+                if (fi.isPrimitive() && (!ei.isStandaloneExpression() || !JavaParserFacade.get(typeSolver).getType(ei).isPrimitive())) {
                     return Optional.empty();
                 }
             }
