@@ -1,35 +1,45 @@
 /*
- * Copyright 2016 Federico Tomassetti
+ * Copyright (C) 2015-2016 Federico Tomassetti
+ * Copyright (C) 2017-2024 The JavaParser Team.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * This file is part of JavaParser.
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * JavaParser can be used either under the terms of
+ * a) the GNU Lesser General Public License as published by
+ *     the Free Software Foundation, either version 3 of the License, or
+ *     (at your option) any later version.
+ * b) the terms of the Apache License
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * You should have received a copy of both licenses in LICENCE.LGPL and
+ * LICENCE.APACHE. Please refer to those files for details.
+ *
+ * JavaParser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
  */
 
 package com.github.javaparser.symbolsolver.model.typesystem;
 
-import com.github.javaparser.resolution.declarations.ResolvedTypeParameterDeclaration;
-import com.github.javaparser.resolution.types.ResolvedTypeVariable;
-import com.github.javaparser.resolution.types.ResolvedWildcard;
-import com.github.javaparser.symbolsolver.model.resolution.TypeSolver;
-import com.github.javaparser.symbolsolver.reflectionmodel.ReflectionClassDeclaration;
-import com.github.javaparser.symbolsolver.resolution.typesolvers.ReflectionTypeSolver;
+import static org.junit.jupiter.api.Assertions.*;
+
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
+import java.util.stream.Collectors;
+
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 
-import java.util.Collections;
-
-import static org.junit.jupiter.api.Assertions.assertEquals;
-import static org.junit.jupiter.api.Assertions.assertTrue;
-import static org.junit.jupiter.api.Assertions.assertThrows;
+import com.github.javaparser.resolution.TypeSolver;
+import com.github.javaparser.resolution.declarations.ResolvedTypeParameterDeclaration;
+import com.github.javaparser.resolution.model.typesystem.ReferenceTypeImpl;
+import com.github.javaparser.resolution.types.ResolvedReferenceType;
+import com.github.javaparser.resolution.types.ResolvedType;
+import com.github.javaparser.resolution.types.ResolvedTypeVariable;
+import com.github.javaparser.resolution.types.ResolvedWildcard;
+import com.github.javaparser.symbolsolver.reflectionmodel.ReflectionClassDeclaration;
+import com.github.javaparser.symbolsolver.resolution.typesolvers.ReflectionTypeSolver;
 
 class WildcardUsageTest {
 
@@ -58,10 +68,10 @@ class WildcardUsageTest {
     @BeforeEach
     void setup() {
         typeSolver = new ReflectionTypeSolver();
-        foo = new ReferenceTypeImpl(new ReflectionClassDeclaration(Foo.class, typeSolver), typeSolver);
-        bar = new ReferenceTypeImpl(new ReflectionClassDeclaration(Bar.class, typeSolver), typeSolver);
-        object = new ReferenceTypeImpl(new ReflectionClassDeclaration(Object.class, typeSolver), typeSolver);
-        string = new ReferenceTypeImpl(new ReflectionClassDeclaration(String.class, typeSolver), typeSolver);
+        foo = new ReferenceTypeImpl(new ReflectionClassDeclaration(Foo.class, typeSolver));
+        bar = new ReferenceTypeImpl(new ReflectionClassDeclaration(Bar.class, typeSolver));
+        object = new ReferenceTypeImpl(new ReflectionClassDeclaration(Object.class, typeSolver));
+        string = new ReferenceTypeImpl(new ReflectionClassDeclaration(String.class, typeSolver));
         superFoo = ResolvedWildcard.superBound(foo);
         superBar = ResolvedWildcard.superBound(bar);
         extendsFoo = ResolvedWildcard.extendsBound(foo);
@@ -420,4 +430,57 @@ class WildcardUsageTest {
         assertEquals(false, ref.getFieldType("bar").isPresent());
     }*/
 
+//    @Test
+//	void testIsAssignableByGenerics() {
+//		ResolvedType listOfSomethingExtendingNumbers = genericType(List.class.getCanonicalName(),
+//				extendsBound(Number.class.getCanonicalName()));
+//		ResolvedType listOfNumbers = genericType(List.class.getCanonicalName(), Number.class.getCanonicalName());
+//		ResolvedType listOfSomethingExtendingIntegers = genericType(List.class.getCanonicalName(),
+//				extendsBound(Integer.class.getCanonicalName()));
+//
+//		ResolvedType list1 = genericType(List.class.getCanonicalName(), Integer.class.getCanonicalName());
+//
+//		print(listOfSomethingExtendingIntegers.asReferenceType().getAllAncestors());
+//
+//		Collection<? extends Number> c1;
+//		Collection<Number> c2 = new ArrayList<>();;
+//		List<? extends Number> lnum = new ArrayList<>();
+//		List<? extends Integer> lint = new ArrayList<>();
+//		c1 = lint;
+//		c1 = lnum;
+//		lnum = lint;
+//		c1 = c2;
+//	}
+
+
+    // Utility methods
+
+	private void print(List<ResolvedReferenceType> ancestors) {
+		for (ResolvedReferenceType ancestor : ancestors) {
+			System.out.println(ancestor.describe());
+		}
+	}
+
+	private List<ResolvedType> types(String... types) {
+		return Arrays.stream(types).map(type -> type(type)).collect(Collectors.toList());
+	}
+
+	private ResolvedType type(String type) {
+		return new ReferenceTypeImpl(typeSolver.solveType(type));
+	}
+
+	private ResolvedType genericType(String type, String... parameterTypes) {
+		return new ReferenceTypeImpl(typeSolver.solveType(type), types(parameterTypes));
+	}
+
+	private ResolvedType genericType(String type, ResolvedType... parameterTypes) {
+		return new ReferenceTypeImpl(typeSolver.solveType(type), Arrays.asList(parameterTypes));
+	}
+
+	private ResolvedType extendsBound(String type) {
+		return ResolvedWildcard.extendsBound(type(type));
+	}
+
+
+
 }
