@@ -1,7 +1,28 @@
+/*
+ * Copyright (C) 2015-2016 Federico Tomassetti
+ * Copyright (C) 2017-2024 The JavaParser Team.
+ *
+ * This file is part of JavaParser.
+ *
+ * JavaParser can be used either under the terms of
+ * a) the GNU Lesser General Public License as published by
+ *     the Free Software Foundation, either version 3 of the License, or
+ *     (at your option) any later version.
+ * b) the terms of the Apache License
+ *
+ * You should have received a copy of both licenses in LICENCE.LGPL and
+ * LICENCE.APACHE. Please refer to those files for details.
+ *
+ * JavaParser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ */
+
 package com.github.javaparser.symbolsolver.resolution.naming;
 
-import com.github.javaparser.*;
-
+import com.github.javaparser.ParseStart;
+import com.github.javaparser.StaticJavaParser;
 import com.github.javaparser.ast.CompilationUnit;
 import com.github.javaparser.ast.Node;
 import com.github.javaparser.ast.body.ClassOrInterfaceDeclaration;
@@ -283,6 +304,13 @@ class NameLogicTest extends AbstractNameLogicTest {
                 NameCategory.TYPE_NAME, ParseStart.CLASS_BODY);
     }
 
+    @Test
+    void instanceOfPatternTypeName() {
+        // Note: Requires JDK14
+        assertNameInCodeIsSyntactically("void myMethod() { if (myValue instanceof Foo f) { }; }", "Foo",
+                NameCategory.TYPE_NAME, ParseStart.CLASS_BODY);
+    }
+
     @Test
     void methodReferenceTypeName() {
         assertNameInCodeIsSyntactically("void myMethod() { Object o = Foo::myMethod; }", "Foo",
@@ -795,6 +823,13 @@ class NameLogicTest extends AbstractNameLogicTest {
                 REFERENCE, ParseStart.CLASS_BODY);
     }
 
+    @Test
+    void classifyRoleInstanceOfPatternTypeName() {
+        // Note: Requires JDK14
+        assertNameInCodeHasRole("void myMethod() { if (myValue instanceof Foo f) { }; }", "Foo",
+                REFERENCE, ParseStart.CLASS_BODY);
+    }
+
     @Test
     void classifyRoleMethodReferenceTypeName() {
         assertNameInCodeHasRole("void myMethod() { Object o = Foo::myMethod; }", "Foo",
@@ -922,5 +957,5 @@ class NameLogicTest extends AbstractNameLogicTest {
                         "public static void travelThroughTime(Date destination) {  }",
                 "date", DECLARATION, ParseStart.CLASS_BODY);
     }
-    
+
 }
